package com.ibm.stg.rtc.ext.common;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.common.internal.workflow.WorkflowInfo;
import com.ibm.team.workitem.common.model.IWorkItem;

public class ChildWorkItemTypeStatesHelper {
	
	private static final String TAG_REQUIRED_CHILD_TYPE = "requiredChildType";
	private static final String TAG_STATE = "state";
	private static final String ATTR_CHILD_WORKITEM_TYPE = "childWorkItemType";
	
	private static final String ATTR_ALLOW_NO_CHILDREN = "allowNoChildren";
	private static final String ATTR_REQUIRED_STATE = "requiredState";
	
	
	/**
	 * @param allChildren
	 * @param childType
	 * @return
	 */
	public static List<IWorkItem> getChildrenOfType(List<IWorkItem> allChildren, String childType) {
		List<IWorkItem> children = new ArrayList<IWorkItem>();
		for(IWorkItem child : allChildren){
			if(child.getWorkItemType().equals(childType)){
				children.add(child);
			}
		}
		return children;
	}

	/**
	 * @param conditionElement
	 * @return a map of child type id to the set of states
	 * @throws TeamRepositoryException 
	 */
	public static HashSet<ChildWorkItemTypeStates> getChildTypesAndStates(IProcessConfigurationElement conditionElement) throws TeamRepositoryException {
		HashSet<ChildWorkItemTypeStates> childWorkItemTypeStateSet = new HashSet<ChildWorkItemTypeStates>();
		for(IProcessConfigurationElement requiredChildTypeElement : conditionElement.getChildren()){
			if(requiredChildTypeElement.getName().equals(TAG_REQUIRED_CHILD_TYPE)){
				ChildWorkItemTypeStates childStates = new ChildWorkItemTypeStates();
				childStates.childType = requiredChildTypeElement.getAttribute(ATTR_CHILD_WORKITEM_TYPE);
				// check for null or empty type
				if(childStates.childType == null || childStates.childType.length() == 0){
					throw new TeamRepositoryException("Attribute " + ATTR_CHILD_WORKITEM_TYPE + " is required for tag " + TAG_REQUIRED_CHILD_TYPE);
				}
				childStates.allowNoChildren = ConfigurationHelper.getBooleanAttributeValue(requiredChildTypeElement, ATTR_ALLOW_NO_CHILDREN);
				
				List<String> stateIds = ConfigurationHelper.getChildrenTagIds(requiredChildTypeElement, TAG_STATE);		
				
				//retrieve the state element children to inspect if the child state is required
				List<String> requiredStateIds = new ArrayList<String>();
				for (IProcessConfigurationElement stateElement : requiredChildTypeElement.getChildren()) {					
					boolean isStateRequired = ConfigurationHelper.getBooleanAttributeValue(stateElement, ATTR_REQUIRED_STATE);
					if (isStateRequired) {
						requiredStateIds.add(stateElement.getAttribute(ConfigurationHelper.ATTR_ID));
					}
				}
				
				childStates.childStates = convertStateIds(stateIds);
				childStates.requiredChildStates = convertStateIds(requiredStateIds);
				childWorkItemTypeStateSet.add(childStates);
			}
		}
		return childWorkItemTypeStateSet;
	}
	
	// convert state ids to their legacy ids. if this is not done, a state id of "s2" will return the string
	// identifier of "2" which does not match the configured states. so this method removes the "s" prefix
	// to legacy states. see com.ibm.team.workitem.common.internal.workflow.WorkflowInfo
	private static List<String> convertStateIds(List<String> stateIds) {
		List<String> convertedStateIds = new ArrayList<String>();
		for(String stateId : stateIds){
			convertedStateIds.add(WorkflowInfo.stripOffPrefix(stateId, 's'));
		}
		return convertedStateIds;
	}
}
