package com.ibm.stg.rtc.ext.common;

import java.util.ArrayList;
import java.util.List;

import com.ibm.team.process.common.IProcessConfigurationElement;

/**
 * @author Kyle Christianson
 *
 */
public class InternalRelease {

	public static final String ATTR_IN_SERVICE = "inService";
	public static final String ATTR_SOURCE_TYPE = "sourceType";

	private String name;
	private String id;
	private boolean inService;
	private String sourceControlType;

	private List<Stream> streams;

	private IProcessConfigurationElement element;

	public InternalRelease(IProcessConfigurationElement element){
		if(element == null){
			throw new IllegalArgumentException("InternalRelease constructor cannot accept a null element");
		}
		this.element = element;
		this.streams = new ArrayList<Stream>();
		this.name = getRequiredAttribute(element, ConfigurationHelper.ATTR_NAME);
		this.id = getRequiredAttribute(element, ConfigurationHelper.ATTR_ID);
		this.inService = ConfigurationHelper.getBooleanAttributeValue(element, ATTR_IN_SERVICE);
		this.sourceControlType = getRequiredAttribute(element, ATTR_SOURCE_TYPE);

		for(IProcessConfigurationElement child : element.getChildren()){
			if(child.getName().equals(ConfigurationHelper.TAG_STREAM)){
				Stream stream = new Stream(this, child);
				streams.add(stream);
			}
		}

	}

	private String getRequiredAttribute(IProcessConfigurationElement element, String attribute){
		String value = element.getAttribute(attribute);
		if(value == null || value.length() == 0){
			throw new IllegalStateException("Each internalRelease element must contain a '" + attribute + "' attribute.");
		}

		return value;
	}

	public String getName() {
		return name;
	}

	public String getIdentifier() {
		return id;
	}

	public boolean isInService() {
		return inService;
	}

	public Stream[] getStreams() {
		return streams.toArray(new Stream[streams.size()]);
	}

	public String getSourceControlType() {
		return sourceControlType;
	}

}
