package com.ibm.stg.rtc.ext.common;

import java.util.List;

import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IRole;
import com.ibm.team.process.service.IProcessServerService;
import com.ibm.team.process.service.IServerProcess;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.service.IPermissionService;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;

public class PermissionsHelper {

	public static boolean isUserSTGAdmin(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService) {
		return isUserRole(saveParameter, contributor, processServerService, "STGAdmin");
	}

	public static boolean isUserProductOwner(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService) {
		return isUserRole(saveParameter, contributor, processServerService, "Product Owner");
	}

	public static boolean isUserScrumMaster(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService) {
		return isUserRole(saveParameter, contributor, processServerService, "Scrum Master");
	}

	public static boolean isUserTeamMember(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService) {
		return isUserRole(saveParameter, contributor, processServerService, "Team Member");
	}

	public static boolean isUserStakeholder(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService) {
		return isUserRole(saveParameter, contributor, processServerService, "Stakeholder");
	}

	public static boolean isUserAssignedRole(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService) {
		try {
			IProcessArea processArea = saveParameter.getNewProcessArea();
			if (processArea == null) {
				processArea = saveParameter.getOldProcessArea();
			}
			IServerProcess serverProcess = processServerService.getServerProcess(processArea);
			IRole[] roles = serverProcess.getContributorRoles(contributor, processArea);
			if (roles.length > 1) {
				return true;
			}
		} catch (TeamRepositoryException e) {
			e.printStackTrace();
		}
		return false;
	}

	public static boolean canUserDeleteSubscriber(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService, IPermissionService permissionService) {
		if (isUserSTGAdmin(saveParameter, contributor, processServerService)) {
			return true;
		} else if (isUserProductOwner(saveParameter, contributor, processServerService)) {
			return true;
		} else if (isUserScrumMaster(saveParameter, contributor, processServerService)) {
			return true;
		} else if (isUserJazzAdmin(saveParameter, contributor, processServerService, permissionService)) {
			return true;
		} else {
			return false;
		}
	}

	public static boolean isUserJazzAdmin(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService, IPermissionService permissionService) {
		if (isUserSTGAdmin(saveParameter, contributor, processServerService)) {
			try {
				permissionService.assertPermission(IPermissionService.JAZZ_ADMINS);
				return true;
			} catch (PermissionDeniedException e) {
				return false;
			}
		}
		return false;
	}

	private static boolean isUserRole(ISaveParameter saveParameter, IContributor contributor, IProcessServerService processServerService, String roleName) {
		boolean isUserSTGAdmin = false;
		try {
			IProcessArea processArea = saveParameter.getNewProcessArea();
			if (processArea == null) {
				processArea = saveParameter.getOldProcessArea();
			}
			IServerProcess serverProcess = processServerService.getServerProcess(processArea);
			IRole[] roles = serverProcess.getContributorRoles(contributor, processArea);
			for (int r = 0; r < roles.length; r++) {
				IRole role = roles[r];
				if (role.getId().equals(roleName)) {
					return true;
				}
			}
		} catch (TeamRepositoryException e) {
			e.printStackTrace();
		}
		return isUserSTGAdmin;
	}

	public static boolean isUserRole(IAuditableCommon auditableCommon, IProcessArea processArea, String strRole) throws TeamRepositoryException{
		List<IRole> roles = auditableCommon.getProcess(processArea, null).getContributorRoles(auditableCommon.getUser(), processArea, null);
		for(IRole role : roles){
			if(role.getId().equals(strRole)){
				return true;
			}
		}
		return false;
	}

}
