package com.ibm.stg.rtc.ext.common;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.links.common.IItemReference;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.common.registry.IEndPointDescriptor;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.internal.model.WorkItemAttributes;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.IState;
import com.ibm.team.workitem.common.model.ISubscriptions;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.common.model.IWorkItemReferences;
import com.ibm.team.workitem.common.model.IWorkItemType;
import com.ibm.team.workitem.common.model.Identifier;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;
import com.ibm.team.workitem.common.workflow.IWorkflowAction;
import com.ibm.team.workitem.common.workflow.IWorkflowInfo;
import com.ibm.team.workitem.service.IWorkItemServer;


public class WorkItemCommonTasks {

	/**
	 * Returns an array of workItem's child work items of type childTypeId
	 * @param childTypeId The type id of children to return. null to return all child work items. Ex: WorkItemCommonIDs.CHANGE_RECORD_ID
	 * @param workItem The work item to find the children of
	 * @param auditableCommon
	 * @param workItemCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getChildWorkItems(String childTypeId, IWorkItem workItem, IAuditableCommon auditableCommon, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		if(workItem == null){
			return null;
		}
		IWorkItemReferences references = workItemCommon.resolveWorkItemReferences(workItem, monitor);
		return getChildWorkItems(childTypeId, references, auditableCommon, monitor);
	}
	
	//STG Defect 14807 - retrieve references instead of work items
	public List<IReference> getAllChildWorkItemReferences(IWorkItem workItem, IAuditableCommon auditableCommon, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException {
		if(workItem == null){
			return null;
		}
		IWorkItemReferences references = workItemCommon.resolveWorkItemReferences(workItem, monitor);
		return getAllChildWorkItemReferences(references, auditableCommon, workItemCommon, monitor);
	}
	
	//STG Defect 14807 - retrieve references instead of work items
	public List<IReference> getAllChildWorkItemReferences(IWorkItemReferences references, IAuditableCommon auditableCommon, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException {		
		return references.getReferences(WorkItemEndPoints.CHILD_WORK_ITEMS);
	}
		
	/**
	 * Returns an array of the child work items in references of type childTypeId
	 * @param childTypeId The type id of children to return. null to return all child work items. Ex: WorkItemCommonIDs.CHANGE_RECORD_ID
	 * @param references All the references of the work item
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getChildWorkItems(String childTypeId, IWorkItemReferences references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException {
		if(childTypeId == null){
			return null;
		}

		// Get the references to other items related to this work item, in this case child work items
		List<IReference> workItemReferences = references.getReferences(WorkItemEndPoints.CHILD_WORK_ITEMS);
		return getWorkItemsFromReferences(childTypeId, workItemReferences, auditableCommon, monitor);
	}

	/**
	 * Returns the work item from the duplicate of link
	 * @param references
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public IWorkItem getDuplicateOfWorkItem(IWorkItemReferences references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException{

		// Get the references to the duplicate work item, there should only be one
		List<IReference> workItemReferences = references.getReferences(WorkItemEndPoints.DUPLICATE_OF_WORK_ITEM);
		List<IWorkItem> workItems = getWorkItemsFromReferences(workItemReferences, auditableCommon, monitor);

		if(workItems == null || workItems.size() == 0){
			return null;
		}else{
			return workItems.get(0);
		}
	}

	/**
	 * Returns the work item from the duplicate of link
	 * @param duplicateTypeId
	 * @param references
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public IWorkItem getDuplicateOfWorkItem(String duplicateTypeId, IWorkItemReferences references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		if(duplicateTypeId == null){
			return null;
		}

		// Get the references to the duplicate work item, there should only be one
		List<IReference> workItemReferences = references.getReferences(WorkItemEndPoints.DUPLICATE_OF_WORK_ITEM);
		List<IWorkItem> workItems = getWorkItemsFromReferences(duplicateTypeId, workItemReferences, auditableCommon, monitor);

		if(workItems == null || workItems.size() == 0){
			return null;
		}else{
			return workItems.get(0);
		}
	}

	/**
	 * Returns true if a duplicate of link exists with the given duplicateWIType.
	 * If duplicateWIType is null, a duplicate of link of any type will result true.
	 * @param references
	 * @param duplicateWIType
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public boolean hasDuplicateOfLink(IWorkItemReferences  references, String duplicateWIType, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		IWorkItem duplicateOfWI;
		if(duplicateWIType == null){
			duplicateOfWI = this.getDuplicateOfWorkItem(references, auditableCommon, monitor);
		}else{
			duplicateOfWI = this.getDuplicateOfWorkItem(duplicateWIType, references, auditableCommon, monitor);
		}

		if(duplicateOfWI == null){
			return false;
		}else{
			return true;
		}
	}

	/**
	 * Returns true if the references contain a new or changed duplicate of link
	 * @param references
	 * @return
	 * @throws TeamRepositoryException
	 */
	public boolean hasNewDuplicateOfLink(IWorkItemReferences  references) throws TeamRepositoryException{
		List<IEndPointDescriptor> changed = references.getChangedReferenceTypes();
		if(changed.contains(WorkItemEndPoints.DUPLICATE_OF_WORK_ITEM)){
			List<IReference> duplicates = references.getReferences(WorkItemEndPoints.DUPLICATE_OF_WORK_ITEM);
			if(duplicates.size() > 0){
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns the work item from the duplicated by link
	 * @param references
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getDuplicateWorkItems(IWorkItemReferences references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException{

		// Get the references to the duplicate work item, there should only be one
		List<IReference> workItemReferences = references.getReferences(WorkItemEndPoints.DUPLICATE_WORK_ITEM);
		List<IWorkItem> workItems = getWorkItemsFromReferences(workItemReferences, auditableCommon, monitor);

		return workItems;
	}

	/**
	 * Returns true if the references contain a new or changed duplicate link
	 * @param references
	 * @return
	 * @throws TeamRepositoryException
	 */
	public boolean hasNewDuplicateLink(IWorkItemReferences  references) throws TeamRepositoryException{
		List<IEndPointDescriptor> changed = references.getChangedReferenceTypes();
		if(changed.contains(WorkItemEndPoints.DUPLICATE_WORK_ITEM)){
			List<IReference> duplicates = references.getReferences(WorkItemEndPoints.DUPLICATE_WORK_ITEM);
			if(duplicates.size() > 0){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns true if the references contain a new or changed parent link
	 * @param references
	 * @return
	 * @throws TeamRepositoryException
	 */
	public boolean hasNewParentLink(IWorkItemReferences references) throws TeamRepositoryException{
		List<IEndPointDescriptor> changed = references.getChangedReferenceTypes();
		if(changed.contains(WorkItemEndPoints.PARENT_WORK_ITEM)){
			return !references.getReferences(WorkItemEndPoints.PARENT_WORK_ITEM).isEmpty();
		}
		return false;
	}
	
	/**
	 * Returns true if the references contain a new or changed child link
	 * @param references
	 * @return
	 * @throws TeamRepositoryException
	 */
	public boolean hasNewChildLink(IWorkItemReferences references) throws TeamRepositoryException{
		List<IEndPointDescriptor> changed = references.getChangedReferenceTypes();
		if(changed.contains(WorkItemEndPoints.CHILD_WORK_ITEMS)){
			return !references.getReferences(WorkItemEndPoints.CHILD_WORK_ITEMS).isEmpty();
		}
		return false;
	}

	/**
	 * Returns an array of workItem's child work items
	 * @param workItem The work item to find the children of
	 * @param auditableCommon
	 * @param workItemCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getAllChildWorkItems(IWorkItem workItem, IAuditableCommon auditableCommon, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException{		
		List<IReference> references = getAllChildWorkItemReferences(workItem, auditableCommon, workItemCommon, monitor);
		return getWorkItemsFromReferences(references, auditableCommon, monitor);
	}

	/**
	 * Returns an array of the child work items in references
	 * @param references All the references of the work item
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getAllChildWorkItems(IWorkItemReferences references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException {
		// Get the references to other items related to this work item, in this case child work items
		List<IReference> workItemReferences = references.getReferences(WorkItemEndPoints.CHILD_WORK_ITEMS);
		return getWorkItemsFromReferences(workItemReferences, auditableCommon, monitor);
	}

	/**
	 * Returns the parent work item of workItem or null if a parent work item does not exist.
	 * @param workItem
	 * @param auditableCommon
	 * @param workItemCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public IWorkItem getParentWorkItem(IWorkItem workItem, IAuditableCommon auditableCommon, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		IWorkItemReferences references = workItemCommon.resolveWorkItemReferences(workItem, monitor);
		return getParentWorkItem(references, auditableCommon, monitor);
	}

	/**
	 * Returns the parent work item in the list of references or null if a parent work item does not exist.
	 * @param references
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public IWorkItem getParentWorkItem(IWorkItemReferences references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException {
		IWorkItem parent = null;

		// Get the references to parent items to this work item, hopefully there is only one
		List<IReference> workItemReferences = references.getReferences(WorkItemEndPoints.PARENT_WORK_ITEM);
		if(workItemReferences.size() > 1){
			throw new TeamRepositoryException("Work item has more than one parent");
		}
		else if(workItemReferences.size() == 1){
			parent = getWorkItemsFromReferences(workItemReferences, auditableCommon, monitor).get(0);
		}

		return parent;
	}

	/**
	 * Returns a list of work items which are linked to the given workItem with the given endPointDescriptor.
	 * If the linked object is not a work item, it will be ignored. 
	 * @param workItem
	 * @param endPointDescriptor
	 * @param workItemCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getLinkedWorkItems(IWorkItem workItem, IEndPointDescriptor endPointDescriptor, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		IWorkItemReferences references = workItemCommon.resolveWorkItemReferences(workItem, monitor);
		List<IReference> workItemReferences = references.getReferences(endPointDescriptor);
		return getWorkItemsFromReferences(workItemReferences, workItemCommon.getAuditableCommon(), monitor);
	}

	/**
	 * Resolves the list of references to a list of IWorkItems of the given work item type and returns it
	 * @param workItemType
	 * @param references
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getWorkItemsFromReferences(String workItemType, List<IReference> references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		// get all the work item references
		//added for STG Defect 14807 - retrieve work item from a references
		List<IWorkItem> workItems = getWorkItemsFromReferences(references, auditableCommon, monitor);
		// flip through the work items and remove the ones that are not of the type we are looking for
		ListIterator<IWorkItem> workItemListIterator = workItems.listIterator();
		while(workItemListIterator.hasNext()){		
			IWorkItem workItem = workItemListIterator.next();
			if(!workItem.getWorkItemType().equals(workItemType)){
				workItemListIterator.remove();
			}
		}
		return workItems;
	}	
	
	/**
	 * Resolves the list of references to a list of IWorkItems and returns it
	 * @param references
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public List<IWorkItem> getWorkItemsFromReferences(List<IReference> references, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		List<IWorkItem> workItems = new ArrayList<IWorkItem>();
		//added for STG Defect 14807 - retrieve work item from a references

		// Flip through the references and turn resolve them to IWorkItems
		ListIterator<IReference> workItemListIterator = references.listIterator();
		while (workItemListIterator.hasNext()) {
			IWorkItem workItem = getWorkItemFromReference(workItemListIterator.next(), auditableCommon, monitor);
			if (workItem != null) {
				workItems.add(workItem);
			}
		}

		return workItems;
	}

	/**
	 * Resolves the list of references to a list of IWorkItems and returns it
	 * @param reference
	 * @param profile
	 * @param auditableCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public IWorkItem getWorkItemFromReference(IReference reference, IAuditableCommon auditableCommon,
			IProgressMonitor monitor) throws TeamRepositoryException {
		// resolve the reference to an IWorkItem
		//added for STG Defect 14807 - retrieve work item from a references

		// Check whether this is a reference to an item type, work items are
		// item types if it is not an item reference, skip it
		if (reference.isItemReference()) {
			IItemReference itemReference = (IItemReference) reference;
			IItemHandle itemHandle = itemReference.getReferencedItem();

			// Check whether this is a work item type, if so continue otherwise
			// skip it
			if (itemHandle instanceof IWorkItemHandle) {
				IWorkItemHandle workItemHandle = (IWorkItemHandle) itemHandle;

				// Get the actual details of the work item, in this case the FULL_PROFILE
				return auditableCommon.resolveAuditable(workItemHandle, IWorkItem.FULL_PROFILE, monitor);
			}
		}

		return null;
	}

	/**
	 * Looks for the itemToFind in the itemArray by matching the item id.
	 * Returns the index it was found at or -1 if not found.
	 * Can be used to see if an IWorkItemHandle exists in a list of IWorkItemHandles
	 * @param itemArray
	 * @param itemToFind
	 * @return
	 */
	public int findItem(IItemHandle[] itemArray, IItemHandle itemToFind){
		for(int i = 0; i < itemArray.length; i++){
			if(itemArray[i].sameItemId(itemToFind)){
				return i;
			}
		}
		return -1;
	}

	/**
	 * Looks for the itemToFind in the itemList by matching the item id.
	 * Returns the index it was found at or -1 if not found.
	 * Can be used to see if an IWorkItemHandle exists in a list of IWorkItemHandles
	 * @param itemList
	 * @param itemToFind
	 * @return
	 */
	public int findWorkItem(List<IWorkItem> itemList, IWorkItemHandle itemToFind){
		for(int i = 0; i < itemList.size(); i++){
			if(itemList.get(i).sameItemId(itemToFind)){
				return i;
			}
		}
		return -1;
	}

	/**
	 * Get the workflow action identifier for the specified workflow action ID
	 * @param workflowInfo
	 * @param workflowAction
	 * @return
	 * @throws TeamRepositoryException
	 */
	public Identifier<IWorkflowAction> getActionIdentifierFromString(IWorkflowInfo workflowInfo, String workflowAction) throws TeamRepositoryException {
		if(workflowAction == null)
			return null;

		Identifier<IWorkflowAction>[] allActions = workflowInfo.getAllActionIds();

		for(Identifier<IWorkflowAction> action : allActions){
			if(action.getStringIdentifier().equals(workflowAction)){
				return action;
			}
		}
		throw new TeamRepositoryException("The action '" + workflowAction + "' was not found in the workflow '" + workflowInfo.getName() + "'.");
	}

	/**
	 * Check whether the object is an IWorkItem type
	 * @param obj
	 * @return
	 */
	public boolean isIWorkItemType(Object obj) {
		if (obj instanceof IWorkItem) {
			return true;
		}
		return false;
	}

	public boolean isSaveParamterType(Object obj) {
		if (obj instanceof ISaveParameter) {
			return true;
		}
		return false;
	}

	/**
	 * Checks to see if the attribute not only exists in the workItem, but
	 * also that it has a value.
	 * 
	 * If the attribute is a String, checks to make sure it is not an empty string
	 * otherwise just checks for null
	 * @param workItem
	 * @param attribute
	 * @return
	 * @throws TeamRepositoryException 
	 */
	public boolean attributeHasNonNullValue(IWorkItem workItem, IAttribute attribute, IAuditableCommon auditableCommon) throws TeamRepositoryException{
		if(attribute == null){
			return false;
		}

		// first check to see that the work item even has the attribute
		if(!workItem.isAttributeSet(attribute)){
			return false;
		}else{
			Object value = workItem.getValue(attribute);

			Object nullValue = attribute.getNullValue(auditableCommon, null);

			if(RepositoryCommonTasks.itemsEqual(nullValue, value)){
				return false;
			}else{
				return true;
			}

		}
	}

	/**
	 * checks that the attribute exists on the work item and that the enumeration has a valid value
	 * by checking the value set providers
	 * 
	 * @param workItem
	 * @param attribute
	 * @param auditableCommon
	 * @return
	 * @throws TeamRepositoryException
	 */
	public boolean attributeHasValidValue(IWorkItem workItem, IAttribute attribute, IAuditableCommon auditableCommon) throws TeamRepositoryException{
		if(attribute == null || !workItem.isAttributeSet(attribute)){
			return false;
		}
		Object value = workItem.getValue(attribute);

		Object[] validValues = attribute.getValueSet(auditableCommon, workItem, null);
		for(Object validValue : validValues){
			if(RepositoryCommonTasks.itemsEqual(validValue, value)){
				return true;
			}
		}
		return false;
	}

	/**
	 * Searches the project area for an attribute matching the given attributeId.
	 * This method should be used instead of workItemCommon.findAttribute() because
	 * it will convert public work item attributes to internal ids to find the attribute.
	 * If the attribute cannot be found, or any of the parameters besides monitor are null,
	 * null is returned. 
	 * @param attributeId
	 * @param projectArea
	 * @param workItemCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public IAttribute getAttribute(String attributeId, IProjectAreaHandle projectArea, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		if(attributeId == null || projectArea == null || workItemCommon == null){
			return null;
		}
		// first try to retrieve the attribute as it is
		IAttribute attribute = workItemCommon.findAttribute(projectArea, attributeId, monitor);
		if(attribute == null){
			// the attribute was not found, so try converting the work item attribute id to the built in internal id and try again
			// Example: passing "com.ibm.team.workitem.attribute.priority" will return "internalPriority"
			String internalAttributeId = WorkItemAttributes.getAttributeId(Identifier.create(IAttribute.class, attributeId));
			if(attributeId.equals(internalAttributeId) == false){
				// an internal identifier was found for the attribute so try to get the attribute again
				attribute = workItemCommon.findAttribute(projectArea, internalAttributeId, monitor);
			}
		}
		return attribute;
	}

	/**
	 * checks to see if the contributor passed in is the "Unassigned" contributor
	 * @param workItem
	 * @param workItemCommon
	 * @param auditableCommon
	 * @return
	 * @throws TeamRepositoryException
	 */
	public boolean isUnassignedContributor(IContributorHandle contributor, IAuditableCommon auditableCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		if (contributor == null)
			return false;

		// first find the owner attribute
		IAttribute ownerAttr = null;
		List<IAttribute> allAttributes = auditableCommon.findAuditables(IAttribute.SMALL_PROFILE, monitor);
		for (IAttribute attribute: allAttributes) {
			if (IWorkItem.OWNER_PROPERTY.equals(attribute.getIdentifier())){
				ownerAttr = attribute;
				break;
			}
		}

		// the owner attribute should always be found
		if(ownerAttr == null){
			throw new TeamRepositoryException("IWorkItem.OWNER_PROPERTY attribute could not be found");
		}

		// get the unassigned contributor by calling getNullValue method
		Object nullval = ownerAttr.getNullValue(auditableCommon, monitor);

		// compare the contributor with the unassigned contributor
		if (contributor == nullval)
			return true;

		if (nullval instanceof IItemHandle)
			return (contributor.sameItemId((IItemHandle) nullval));

		return contributor.equals(nullval);
	}

	/**
	 * Subscribes the contributor to the work item.
	 * 
	 * This method performs a save operation which will cause preconditions and follow-up actions to run.
	 * 
	 * @param workItemId
	 * @param contributor
	 * @param workItemServer
	 * @throws TeamRepositoryException
	 */
	public void subscribeContributor(int workItemId, IContributorHandle contributorHandle, IWorkItemServer workItemServer, IProgressMonitor monitor) throws TeamRepositoryException{
		if(contributorHandle == null){
			return;
		}

		// do not subscribe the unassigned contributor
		if(this.isUnassignedContributor(contributorHandle, workItemServer.getAuditableCommon(), monitor)){
			return;
		}

		// get a fresh working copy
		IWorkItem workItemWC = (IWorkItem)workItemServer.findWorkItemById(workItemId, IWorkItem.FULL_PROFILE, null).getWorkingCopy();
		ISubscriptions subscriptions = workItemWC.getSubscriptions();
		if(!subscriptions.contains(contributorHandle)){
			subscriptions.add(contributorHandle);
			workItemServer.saveWorkItem2(workItemWC, null, null);
		}
	}

	public static String findActionFromStates(String targetState, String fromState, IWorkflowInfo workflowInfo){
		if(fromState == null){
			return workflowInfo.getStartActionId().getStringIdentifier();
		}
		Identifier<IWorkflowAction>[] possibleActions = workflowInfo.getActionIds(Identifier.create(IState.class, fromState));
		for(Identifier<IWorkflowAction> action : possibleActions){
			if(workflowInfo.getActionResultState(action).getStringIdentifier().equals(targetState)){
				return action.getStringIdentifier();
			}
		}
		return null;
	}

	/**
	 * Returns the work item type's display name if it exists, otherwise returns the type's string identifier
	 * @param workItem
	 * @param workItemCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public String getTypeName(IWorkItem workItem, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		IWorkItemType workItemType = workItemCommon.findWorkItemType(workItem.getProjectArea(), workItem.getWorkItemType(), monitor);
		String typeName;
		if(workItemType != null){
			typeName = workItemType.getDisplayName();
		}else{
			typeName = workItem.getWorkItemType();
		}

		return typeName;
	}

	/**
	 * returns the work item state's display name if it exists, otherwise returns the state's string identifier
	 * @param workItem
	 * @param workItemCommon
	 * @param monitor
	 * @return
	 * @throws TeamRepositoryException
	 */
	public String getStateName(IWorkItem workItem, IWorkItemCommon workItemCommon, IProgressMonitor monitor) throws TeamRepositoryException{
		IWorkflowInfo workflowInfo = workItemCommon.findWorkflowInfo(workItem, monitor);
		String stateName = null;
		if(workflowInfo != null){
			stateName = workflowInfo.getStateName(workItem.getState2());
		}
		if(stateName == null){
			stateName = workItem.getState2().getStringIdentifier();
		}

		return stateName;
	}
}
