package com.ibm.stg.rtc.ext.stgdefect.advisors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import com.ibm.stg.rtc.ext.common.ConfigurationHelper;
import com.ibm.stg.rtc.ext.common.InternalRelease;
import com.ibm.stg.rtc.ext.common.ItemHandleKey;
import com.ibm.stg.rtc.ext.common.RepositoryCommonTasks;
import com.ibm.stg.rtc.ext.common.SCMHelper;
import com.ibm.stg.rtc.ext.common.STGCommonIDs;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.filesystem.common.workitems.internal.IFilesystemWorkItemService;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.common.service.ILinkService;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.TeamOperationCanceledException;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.IWorkspaceHandle;
import com.ibm.team.scm.common.process.IScmDeltaSource;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.ILiteral;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.common.model.IWorkItemReferences;
import com.ibm.team.workitem.common.model.Identifier;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;
import com.ibm.team.workitem.common.model.WorkItemLinkTypes;
import com.ibm.team.workitem.service.IWorkItemServer;

/**
 * @author christik
 *
 */
public class CreateCROnDeliverAdvisor extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.CreateCROnDeliverAdvisor";
	public static final String NAME = "STG: Create CR On Deliver";

	public void run(AdvisableOperation operation, IProcessConfigurationElement participantConfig, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {

		IScmDeltaSource scmDeltaSource;

		Object tempObj = operation.getOperationData();
		if (tempObj instanceof IScmDeltaSource) {
			scmDeltaSource = (IScmDeltaSource) tempObj;
		} else {
			return;
		}

		IWorkItemServer workItemServer = this.getService(IWorkItemServer.class);

		List<String> workItemTypesConfigured = ConfigurationHelper.getWITypesConfigured(participantConfig);

		IProjectAreaHandle projectAreaHandle = operation.getProcessArea().getProjectArea();

		// first collect all the change records which are needed
		ChangeRecordWrapper[] changeRecordsNeeded = getAllChangeRecordsNeeded(workItemTypesConfigured, scmDeltaSource, projectAreaHandle, monitor);

		// next find existing change records and add them to the wrappers, we don't want to create a change record if it already exists
		addExistingChangeRecords(changeRecordsNeeded, workItemServer, monitor);

		// now try to create all the change records that do not already exist
		createMissingChangeRecords(changeRecordsNeeded, workItemServer, projectAreaHandle, collector, monitor);

		// finally, link all change sets to the change records that were created
		linkChangeSetsToChangeRecords(changeRecordsNeeded, collector, monitor);

	}

	/**
	 * @param changeRecordsNeeded
	 * @param filesystemWorkItemService
	 * @throws TeamRepositoryException 
	 */
	private void linkChangeSetsToChangeRecords(ChangeRecordWrapper[] changeRecordsNeeded, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		IFilesystemWorkItemService filesystemWorkItemService = getService(IFilesystemWorkItemService.class);
		for(ChangeRecordWrapper wrapper : changeRecordsNeeded){
			if(wrapper.changeRecord != null){
				for(IChangeSetHandle changeSet : wrapper.changeSets){
					try{
						// the following method will only create the link if a link between the change record and change set does not already exist
						// we might gain some performance by caching this information from earlier and skipping this call
						filesystemWorkItemService.createLink(wrapper.repositoryStream, changeSet, new IWorkItemHandle[]{wrapper.changeRecord}, null);
					}catch(TeamOperationCanceledException e){
						RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
						IReportInfo info = collector.createProblemInfo("Failed to link Change Record", "The follow-up action could not link the change set to Change Record " + wrapper.changeRecord.getId() +
								" because a precondition failed with the following message: \n" + repoCommon.getFirstErrorDescription(e), ID);
						// add this info as a WARNING and suppress the save work item errors so the deliver operation can continue
						info.setSeverity(IProcessReport.WARNING);
						collector.addInfo(info);
						repoCommon.suppressAllErrors(NAME, e);
					}
				}
			}
		}
	}

	private void createMissingChangeRecords(ChangeRecordWrapper[] changeRecordsNeeded, IWorkItemServer workItemServer, IProjectAreaHandle projectAreaHandle, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		for(ChangeRecordWrapper wrapper : changeRecordsNeeded){
			if(wrapper.changeRecord == null){
				try{
					wrapper.changeRecord = createChangeRecord(workItemServer, wrapper.parentWorkItem, wrapper.internalRelease, monitor);
				} catch (TeamOperationCanceledException e) {
					RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
					IReportInfo info = collector.createProblemInfo("Failed to create Change Record", NAME + " failed to create a Change Record for parent work item " + wrapper.parentWorkItem.getId() + 
							" with Internal Release '" + wrapper.internalRelease.getName() + 
							"' because a precondition failed with the following message: \n" + repoCommon.getFirstErrorDescription(e), ID);
					info.setProblemObject(wrapper.parentWorkItem.getItemHandle());
					// add this info as a WARNING and suppress the save work item errors so the deliver operation can continue
					info.setSeverity(IProcessReport.WARNING);
					collector.addInfo(info);
					repoCommon.suppressAllErrors(NAME, e);
				}
			}
		}
	}

	private void addExistingChangeRecords(ChangeRecordWrapper[] changeRecordsNeeded, IWorkItemServer workItemServer, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		for(ChangeRecordWrapper wrapper : changeRecordsNeeded){
			// use the parent work item's project area, it is not necessarily the same as the operation's project area (the stream owner)
			IProjectAreaHandle projectAreaHandle = wrapper.parentWorkItem.getProjectArea();

			//STG Defect 14807: this will not fail if the user does not have permission to child work item
			List<IWorkItem> existingChangeRecords = new ArrayList<IWorkItem>();
			List<IReference> childReferences = workItemCommonTasks.getAllChildWorkItemReferences(wrapper.parentWorkItem, workItemServer.getAuditableCommon(), workItemServer, monitor);
			for (IReference childReference : childReferences) {
				try {
					IWorkItem workItem = workItemCommonTasks.getWorkItemFromReference(childReference,
							workItemServer.getAuditableCommon(), monitor);
					if (workItem.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID)) {
						existingChangeRecords.add(workItem);	
					}
				}
				catch (PermissionDeniedException e) {
					//code is only concerned with child change records in the same project
					//intentional no-op
				}
			}
			
			for(IWorkItem existingChangeRecord : existingChangeRecords){

				// STG Defect 13057 - do not consider change records in different project areas
				if(!existingChangeRecord.getProjectArea().sameItemId(projectAreaHandle)){
					continue;
				}

				if(SCMHelper.sameInternalRelease(existingChangeRecord, wrapper.internalRelease, workItemServer, monitor)){
					// found a matching change record!
					wrapper.changeRecord = existingChangeRecord;
					break;
				}
			}
		}
	}

	private ChangeRecordWrapper[] getAllChangeRecordsNeeded(List<String> workItemTypesConfigured, IScmDeltaSource scmDeltaSource, IProjectAreaHandle projectAreaHandle, IProgressMonitor monitor) throws TeamRepositoryException{
		IAuditableCommon auditableCommon = this.getService(IAuditableCommon.class);
		ILinkService linkService = this.getService(ILinkService.class);
		// a change record is needed for each configured work item type associated to a change set + the Internal Release of the stream being delivered to
		// the map's key will contain the parent work item uuid + Internal Release id with no spaces
		Map<String,ChangeRecordWrapper> changeRecordsNeeded = new HashMap<String,ChangeRecordWrapper>();

		Map<ItemHandleKey<IWorkspaceHandle>, List<IChangeSetHandle>> deliverWorkspacesToChangeSets = SCMHelper.getDeliverInformation(scmDeltaSource);

		for (ItemHandleKey<IWorkspaceHandle> workspaceKey : deliverWorkspacesToChangeSets.keySet()) {

			IWorkspaceHandle targetWorkspace = workspaceKey.getItemHandle();
			InternalRelease internalRelease = SCMHelper.getInternalReleaseOfStream(targetWorkspace, projectAreaHandle, auditableCommon);
			if(internalRelease != null){
				List<IChangeSetHandle> allChangeSets = deliverWorkspacesToChangeSets.get(workspaceKey);
				for(IChangeSetHandle changeSet : allChangeSets){
					// this does not return remote change requests!
					List<IWorkItemHandle> associatedWorkItems = SCMHelper.getAssociatedWorkItems(changeSet, linkService);
					for(IWorkItemHandle workItemHandle : associatedWorkItems){
						IWorkItem workItem = auditableCommon.resolveAuditable(workItemHandle, IWorkItem.FULL_PROFILE, monitor);
						// only create change records for the configured work item types
						if(workItemTypesConfigured.contains(workItem.getWorkItemType())){

							// finally all the conditions are met to automatically create/link a change record
							// to this work item and change set so add it to the list to be created
							String key = workItem.getItemId().getUuidValue() + internalRelease.getIdentifier();
							if(changeRecordsNeeded.containsKey(key)){
								changeRecordsNeeded.get(key).changeSets.add(changeSet);
							}else{
								ChangeRecordWrapper wrapper = new ChangeRecordWrapper();
								wrapper.repositoryStream = targetWorkspace;
								wrapper.internalRelease = internalRelease;
								wrapper.parentWorkItem = workItem;
								wrapper.changeSets.add(changeSet);
								changeRecordsNeeded.put(key, wrapper);
							}
						}

					}
				}
			}else{
				// an Internal Release for the stream is not found in metadata so it is not tracked. do nothing
			}

		}

		return changeRecordsNeeded.values().toArray(new ChangeRecordWrapper[changeRecordsNeeded.size()]);
	}

	private IWorkItem createChangeRecord(IWorkItemServer workItemServer, IWorkItem parent, InternalRelease internalRelease, IProgressMonitor monitor) throws TeamRepositoryException{
		// use the parent work item's project area, it is not necessarily the same as the operation's project area (the stream owner)
		IProjectAreaHandle projectAreaHandle = parent.getProjectArea();
		IWorkItem newChangeRecord = workItemServer.createWorkItem2(workItemServer.findWorkItemType(projectAreaHandle, STGDefectCommonIDs.CHANGE_RECORD_ID, monitor));
		// get working copy
		newChangeRecord = (IWorkItem)newChangeRecord.getWorkingCopy();

		// do not set the summary, let the default summary precondition set it
		//newChangeRecord.setHTMLSummary(parent.getHTMLSummary());

		// set parent work item
		IWorkItemReferences references = workItemServer.resolveWorkItemReferences(newChangeRecord, monitor);
		IReference parentReference = WorkItemLinkTypes.createWorkItemReference(parent);
		references.add(WorkItemEndPoints.PARENT_WORK_ITEM, parentReference);

		// set internal release custom attribute
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		IAttribute internalReleaseAttribute = workItemCommonTasks.getAttribute(STGCommonIDs.STG_ATTR_INTERNAL_RELEASE, projectAreaHandle, workItemServer, monitor);
		if(newChangeRecord.hasAttribute(internalReleaseAttribute) == false){
			newChangeRecord.addCustomAttribute(internalReleaseAttribute);
		}
		Identifier<ILiteral> internalReleaseIdentifier = Identifier.create(ILiteral.class, internalRelease.getIdentifier());
		newChangeRecord.setValue(internalReleaseAttribute, internalReleaseIdentifier);


		IStatus result = workItemServer.saveWorkItem2(newChangeRecord, references, null);

		IWorkItem savedWI = workItemServer.getAuditableCommon().resolveAuditable(newChangeRecord, IWorkItem.FULL_PROFILE, monitor);
		return savedWI;
	}


	private class ChangeRecordWrapper{
		IWorkspaceHandle repositoryStream;
		InternalRelease internalRelease;
		IWorkItem parentWorkItem;
		IWorkItem changeRecord;
		List<IChangeSetHandle> changeSets = new ArrayList<IChangeSetHandle>();
	}
}
