package com.ibm.stg.rtc.ext.stgdefect.advisors;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.ConfigurationHelper;
import com.ibm.stg.rtc.ext.common.SCMHelper;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IWorkItem;

/**
 * 
 * @author Karen Paone
 *
 */
public class PreventCRReleaseEditAdvisor extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.PreventCRReleaseEditAdvisor";

	private static final String TAG_ALLOW_CHANGE = "allowChange";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration,
			IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState = null;
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		}
		else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		}
		else {
			return;
		}

		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		}
		else if (tempObj == null) {
			oldWorkItemState = null;
		}
		else {
			return;
		}

		// check to see if the work item is a Change Record, if not return
		if (!(newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID))) {
			return;
		}

		// allow changes and multiples from these configured values
		List<String> allowChangeValues = ConfigurationHelper.getChildrenTagIds(advisorConfiguration, TAG_ALLOW_CHANGE);

		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);

		if (oldWorkItemState != null
				&& !SCMHelper.sameInternalRelease(newWorkItemState, oldWorkItemState, workItemCommon,
						monitor)) {
			// the user is trying to change the internal release

			if (!allowChangeValues.contains(SCMHelper.getInternalReleaseId(oldWorkItemState, workItemCommon,
					monitor))) {
				IAdvisorInfo info = collector
						.createProblemInfo(
								"Invalid Internal Release",
								"Once created, the Internal Release field cannot change on a Change Record. Please 'Cancel' or 'NoChange' this Change Record and open a new one with the correct Internal Release value.",
								ID);
				info.setProblemObject(newWorkItemState.getItemHandle());
				collector.addInfo(info);
				return;
			}
		}
	}
}
