package com.ibm.stg.rtc.ext.stgdefect.advisors;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.links.common.IReference;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;

public class PreventCRRemovalAdvisor  extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.PreventCRRemovalAdvisor";


	/**
	 * Prevents a user from removing the child link to a change record from any work item
	 */
	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IAuditableCommon auditableCommon = this.getService(IAuditableCommon.class);

		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// don't care about new work items because they couldn't have a parent or child to remove
		if(oldWorkItemState == null){
			return;
		}

		// if saving a Change Record, prevent the removal of a parent
		if((newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID))) {
			//STG Defect 14807: use references instead of using retrieving the work item parent
			List<IReference> deletedParent = saveParameter.getNewReferences().getDeletedReferences(WorkItemEndPoints.PARENT_WORK_ITEM);
			if (!deletedParent.isEmpty()) {		       
               IAdvisorInfo info = collector.createProblemInfo("Cannot remove Change Record parent", "You cannot change or remove the parent work item of a Change Record", ID);
               info.setProblemObject(newWorkItemState.getItemHandle());
               collector.addInfo(info);		               
	       }
		}

		// for any other work item type, see if it has and/or had child change records
		//STG Defect 14807: use references instead of using retrieving children work items
       List<IReference> deletedChildren = saveParameter.getNewReferences().getDeletedReferences(WorkItemEndPoints.CHILD_WORK_ITEMS);
       for (IReference childReference : deletedChildren) {
           try {
               IWorkItem childWorkItem = workItemCommonTasks.getWorkItemFromReference(childReference, auditableCommon, monitor);
               if (childWorkItem.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID)) {
                   IAdvisorInfo info = collector.createProblemInfo("Cannot remove Change Record", "You cannot remove child Change Records. Attempted to remove Change Record " + childWorkItem.getId(), ID);
                   info.setProblemObject(newWorkItemState.getItemHandle());
                   collector.addInfo(info);
               }
           }
           catch(PermissionDeniedException e){
               // if the user does not have permission, they will get an RTC error and we won't reach this try/catch
        	   // leaving this in place for best practice
        	   
               // Since this precondition is limited to CRs only, and CRs should only be in the same project area, it might be okay to ignore.
           }
       }
	}
}
