package com.ibm.stg.rtc.ext.stgdefect.advisors;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.ApprovalContainer;
import com.ibm.stg.rtc.ext.common.ApprovalsHelper;
import com.ibm.stg.rtc.ext.common.InternalRelease;
import com.ibm.stg.rtc.ext.common.ItemHandleKey;
import com.ibm.stg.rtc.ext.common.SCMHelper;
import com.ibm.team.links.common.service.ILinkService;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.repository.service.IRepositoryItemService;
import com.ibm.team.scm.common.IChangeSet;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.IWorkspace;
import com.ibm.team.scm.common.IWorkspaceHandle;
import com.ibm.team.scm.common.process.IScmDeltaSource;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IApprovalDescriptor;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.ItemProfile;
import com.ibm.team.workitem.common.model.WorkItemApprovals;

public class PreventDeliverWithoutSTGApprovalAdvisor extends AbstractService implements IOperationAdvisor {
	private static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.PreventDeliverWithoutSTGApprovalAdvisor";
	
	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration,
			IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {	
		if (operation == null) {
			return;
		}
		
		String[] actions = operation.getActions();
		if (actions == null) {
			return;
		}
		
		IScmDeltaSource scmDeltaSource;

		Object tempObj = operation.getOperationData();
		if (tempObj instanceof IScmDeltaSource) {
			scmDeltaSource = (IScmDeltaSource) tempObj;
		} else {
			return;
		}
				
		IAuditableCommon auditableCommon = this.getService(IAuditableCommon.class);
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		ILinkService linkService = this.getService(ILinkService.class);

		//retrieve all of the work items that relate to this delivery
		
		//retrieve everything included in the delivery
		Map<ItemHandleKey<IWorkspaceHandle>, List<IChangeSetHandle>> deliverWorkspacesToChangeSets = SCMHelper.getDeliverInformation(scmDeltaSource);
		
		//go through through the change sets by workspace/stream
		for (ItemHandleKey<IWorkspaceHandle> workspaceKey : deliverWorkspacesToChangeSets.keySet()) {
			IWorkspaceHandle targetWorkspaceHandle = workspaceKey.getItemHandle();
			InternalRelease internalRelease = SCMHelper.getInternalReleaseOfStream(targetWorkspaceHandle, operation.getProcessArea().getProjectArea(), auditableCommon);
			if (internalRelease == null) {
				//not a tracked stream, skip it
				continue;
			}
			
			//retrieve a mapping of the work items that are related to the target deliver stream for the change sets included on deliver
			//if any of these work items does not have the required approval with required state, prevent the deliver
			Map<IChangeSetHandle, List<IWorkItem>> workItemsRelatedToStream = SCMHelper
					.getAssociatedWorkItemsRelatedToStream(deliverWorkspacesToChangeSets.get(workspaceKey),
							targetWorkspaceHandle, workItemCommon, auditableCommon, linkService, monitor);
			
			//each change set needs all qualifying work item with approvals met; if one is missing the approval, fail deliver
			for (IChangeSetHandle changeSetHandle : workItemsRelatedToStream.keySet()) {				
				for (IWorkItem workItem : workItemsRelatedToStream.get(changeSetHandle)) {				
					//determine if there are any approvals required before deliver for this work item type					
					for (ApprovalContainer requiredApproval : ApprovalsHelper.getConfiguredApprovals(advisorConfiguration, workItem, workItemCommon)) {				
						
						//prevent deliver
						if (requiredApproval.hasPreventDeliver()) {
							//determine if the work item has the approval in the state that is required to allowed deliver
							IApprovalDescriptor approvalOnWorkItem = ApprovalsHelper.findApprovalDescriptorByName(workItem, requiredApproval.getName());
							
							if (approvalOnWorkItem == null || !requiredApproval.getRequiredApprovalStatesToDeliver().contains(approvalOnWorkItem.getCumulativeStateIdentifier())) {
								//error - required approval not met 
								IChangeSet changeSet = (IChangeSet) auditableCommon.resolveAuditable(changeSetHandle, ItemProfile
										.createProfile(IChangeSet.ITEM_TYPE, new String[] {IChangeSet.COMMENT_PROPERTY}), monitor);
								IRepositoryItemService itemService = this.getService(IRepositoryItemService.class);
								IWorkspace targetWorkSpace = (IWorkspace) itemService.fetchItem(targetWorkspaceHandle,
										new String[] { IWorkspace.NAME_PROPERTY });

								StringBuilder sb = new StringBuilder();
								Iterator<String> i = requiredApproval.getRequiredApprovalStatesToDeliver().iterator();
								while(i.hasNext()){
									sb.append(WorkItemApprovals.getState(i.next()).getDisplayName());
									if(i.hasNext()){
										sb.append(" or ");
									}
								}
								String requiredStateName = sb.toString();
								
								String errorMsg = "Work Items associated to Change Sets delivered to tracked stream '"
										+ targetWorkSpace.getName()
										+ "' must have "
										+ requiredApproval.getName()
										+ " approval in state "
										+ requiredStateName
										+ ". Change Set "
										+ (changeSet.getComment() == null || changeSet.getComment().trim().equals("") ? "<no comment>"
												: "'" + changeSet.getComment() + "'")
										+ " associated to work item "
										+ workItem.getId()
										+ (approvalOnWorkItem == null ? " does not have "
												+ requiredApproval.getName() + " approval ": " has "
												+ requiredApproval.getName() + " approval in state "
												+ WorkItemApprovals.getState(approvalOnWorkItem.getCumulativeStateIdentifier()).getDisplayName()) + ".";

								IReportInfo info = collector.createProblemInfo("Approval required to deliver",
										errorMsg, ID);
								collector.addInfo(info);
							}						
						}			
					}					
				}
			}
		}
	}

}
