package com.ibm.stg.rtc.ext.stgdefect.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.Identifier;

public class PreventIncorrectFQIRiskAdvisor extends AbstractService implements IOperationAdvisor {
	/*
	 * This precondition prevents the work item from being saved with an incorrect FQI Risk value. A javascript script runs
	 * to dynamically calculate the FQI Risk on the STG Defect form, but users could potentially edit the FQI Risk value
	 * from a query (edit multiple work items function). This precondition will prevent that.
	 */

	public static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.PreventIncorrectFQIRiskAdvisor";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		ISaveParameter saveParameter = null;
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		IWorkItem newWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// check to see if the work item is an STG Defect, if not return
		if(!(newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.STG_DEFECT_ID))){
			return;
		}

		IAttribute fqiCustomerImpact = workItemCommonTasks.getAttribute(STGDefectCommonIDs.STG_DEFECT_ATTR_FQI_CUSTOMER_IMPACT, newWorkItemState.getProjectArea(), workItemCommon, monitor);
		IAttribute fqiCustomerProbability = workItemCommonTasks.getAttribute(STGDefectCommonIDs.STG_DEFECT_ATTR_FQI_CUSTOMER_PROBABILITY, newWorkItemState.getProjectArea(), workItemCommon, monitor);
		IAttribute fqiRisk = workItemCommonTasks.getAttribute(STGDefectCommonIDs.STG_DEFECT_ATTR_FQI_RISK, newWorkItemState.getProjectArea(), workItemCommon, monitor);

		Identifier custImpactValue = (Identifier)newWorkItemState.getValue(fqiCustomerImpact);
		Identifier custProbValue = (Identifier)newWorkItemState.getValue(fqiCustomerProbability);
		int riskValue = (Integer)newWorkItemState.getValue(fqiRisk);

		int calculatedRisk;
		try{
			int impact = getLastIntSegment(custImpactValue);
			int prob = getLastIntSegment(custProbValue);
			calculatedRisk = impact * prob;
		} catch(Exception e){
			calculatedRisk = -1;
		}

		if(riskValue != calculatedRisk){
			// we cannot allow a save if the risk is wrong
			IAdvisorInfo info= collector.createProblemInfo("FQI Risk incorrect", "FQI Risk calculation is incorrect. You cannot manually edit the FQI Risk field.", ID);
			info.setProblemObject(newWorkItemState.getItemHandle());
			collector.addInfo(info);
		}
	}

	private int getLastIntSegment(Identifier id){
		String strId = id.getStringIdentifier();
		int index = strId.lastIndexOf('.');
		return Integer.parseInt(strId.substring(index + 1));
	}

}