package com.ibm.stg.rtc.ext.stgdefect.advisors;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.model.IWorkItem;

public class PreventMoveIfCRActiveAdvisor extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.PreventMoveIfCRActiveAdvisor";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IAuditableCommon auditableCommon = this.getService(IAuditableCommon.class);

		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// When the user is copying (rather than moving) a work item, getOldState() returns
		// null and the precondition is bypassed.
		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// check to see if the user is saving a Change Record
		if(newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID)){
			// do not allow any change records to change projects
			IProjectAreaHandle newProjectArea = newWorkItemState.getProjectArea();
			IProjectAreaHandle oldProjectArea = oldWorkItemState.getProjectArea();

			// if the project areas are different the user is trying to move the change record
			if(!newProjectArea.sameItemId(oldProjectArea)){
				IAdvisorInfo info= collector.createProblemInfo("Cannot move Change Record", "You cannot move a Change Record to a different project. You may wish to cancel all Change Records and move their parent work item.", ID);
				info.setProblemObject(newWorkItemState.getItemHandle());
				collector.addInfo(info);
			}
		}else{
			// for all other work item types, do not allow move if it contains active child change records
			IProjectAreaHandle newProjectArea = newWorkItemState.getProjectArea();
			IProjectAreaHandle oldProjectArea = oldWorkItemState.getProjectArea();

			// if the project areas are different the user is trying to move the work item
			if(!newProjectArea.sameItemId(oldProjectArea)){
				// check for active change records

				List<IWorkItem> changeRecords = workItemCommonTasks.getChildWorkItems(STGDefectCommonIDs.CHANGE_RECORD_ID, saveParameter.getNewReferences(), auditableCommon, monitor);

				for(IWorkItem changeRecord : changeRecords){
					if(STGDefectCommonIDs.CHANGE_RECORD_ACTIVE_STATES.contains(changeRecord.getState2().getStringIdentifier()) || changeRecord.getState2().getStringIdentifier().equals(STGDefectCommonIDs.CHANGE_RECORD_STATE_CLOSED)){
						// the work item cannot be moved because it has active or closed change records
						IAdvisorInfo info= collector.createProblemInfo("Cannot move work item", "You cannot move a work item to a new project if it has active or closed Change Records. You may cancel or nochange all change records to move the work item, or create a new work item in the other project with the copy function.", ID);
						info.setProblemObject(newWorkItemState.getItemHandle());
						collector.addInfo(info);
						break;
					}
				}
			}
		}

	}
}
