package com.ibm.stg.rtc.ext.stgdefect.advisors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.SCMHelper;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.links.common.IReference;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IWorkItem;

/**
 * 
 * @author Karen Paone
 *
 */
public class PreventMultipleCRSameInternalReleaseAdvisor extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.PreventMultipleCRSameInternalReleaseAdvisor";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration,
			IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState = null;
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		}
		else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		}
		else {
			return;
		}
		
		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		}

		// check to see if the work item is a Change Record, if not return
		if (!(newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID))) {
			return;
		}

		IAuditableCommon auditableCommon = this.getService(IAuditableCommon.class);
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);

		//this internal release was not modified on an existing CR, no further checking required
		if (oldWorkItemState != null && SCMHelper.sameInternalRelease(oldWorkItemState, newWorkItemState, workItemCommon, monitor)) {
			return;
		}
		
		// now check that there isn't another change record with the same parent
		// defect
		IWorkItem parent = workItemCommonTasks.getParentWorkItem(saveParameter.getNewReferences(), auditableCommon,
				monitor);
		if (parent == null) {
			// this is an erroneous state that a different precondition will
			// catch
			return;
		}
		//STG Defect 14807: this will not fail if the user does not have permission to child work item
		// code is only concerned with child change records in the same project
		List<IWorkItem> changeRecords = new ArrayList<IWorkItem>();
		List<IReference> childReferences = workItemCommonTasks.getAllChildWorkItemReferences(parent, auditableCommon, workItemCommon, monitor);
		for (IReference childReference : childReferences) {
			try {
				IWorkItem workItem = workItemCommonTasks.getWorkItemFromReference(childReference,
						workItemCommon.getAuditableCommon(), monitor);
				if (workItem.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID)) {
					changeRecords.add(workItem);	
				}
			}
			catch (PermissionDeniedException e) {
				//code is only concerned with child change records in the same project
				//intentional no-op
			}
		}

		// look for a change record with the same internal release
		for (IWorkItem changeRecord : changeRecords) {
			// skip the change record we are saving
			if (changeRecord.sameItemId(newWorkItemState)) {
				continue;
			}

			// skip the change record if the project area is not the same - we
			// are only looking for uniqueness for CRs that are in this project
			if (!changeRecord.getProjectArea().getItemId().equals(parent.getProjectArea().getItemId())) {
				continue;
			}

			// determine if the CRs have the same internal release
			if (SCMHelper.sameInternalRelease(newWorkItemState, changeRecord, workItemCommon, monitor)) {
				// the internal releases are also the same on this CR, error

				// the display values to include in the error message
				String intRel = SCMHelper.getInternalReleaseName(newWorkItemState, workItemCommon, monitor);

				IAdvisorInfo info = collector.createProblemInfo("Invalid Internal Release",
						"There can only be one Change Record per Internal Release. Change Record "
								+ changeRecord.getId() + " already has the Internal Release '" + intRel + "'.", ID);
				info.setProblemObject(newWorkItemState.getItemHandle());
				collector.addInfo(info);
			}
		}
	}
}
