package com.ibm.stg.rtc.ext.stgdefect.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IWorkItem;

/**
 * This advisor prevents change records in a project different from their parent work item
 * from being reopened.
 * 
 * @author Kyle Christianson
 *
 */
public class PreventOrphanedCRReopenAdvisor extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.PreventOrphanedCRReopen";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);

		if (newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID)){
			if(STGDefectCommonIDs.CHANGE_RECORD_ACTION_REOPEN.equals(saveParameter.getWorkflowAction())){
				// the user is performing a reopen on a change record. check to see if it is in a different
				// project are than the parent work item. if so, raise an error
				IWorkItem parent = workItemCommonTasks.getParentWorkItem(saveParameter.getNewReferences(), workItemCommon.getAuditableCommon(), monitor);
				if(parent != null && !parent.getProjectArea().sameItemId(newWorkItemState.getProjectArea())){
					IAdvisorInfo info= collector.createProblemInfo("Cannot Reopen Change Record", "You cannot Reopen a Change Record that exists in a project different from its parent work item. Please create a new Change Record in the parent work item's project.", ID);
					info.setProblemObject(newWorkItemState.getItemHandle());
					collector.addInfo(info);
				}
			}
		}

	}
}
