package com.ibm.stg.rtc.ext.stgdefect.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.Identifier;

public class RequireODCContentTypeAdvisor extends AbstractService implements IOperationAdvisor {

	private static final String ID = "com.ibm.stg.rtc.ext.stgdefect.advisors.RequireODCContentTypeAdvisor";

	private static final String REQUIRED_TARGET_VALUE = "com.ibm.stg.enumeration.odc.target.literal.userDocumentation";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// only applies to STG Defect
		if(!newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.STG_DEFECT_ID)){
			return;
		}

		IAttribute odcContentType = workItemCommonTasks.getAttribute(STGDefectCommonIDs.STG_DEFECT_ATTR_ODC_CONTENT_TYPE, newWorkItemState.getProjectArea(), workItemCommon, monitor);
		IAttribute odcTargetAttr = workItemCommonTasks.getAttribute(STGDefectCommonIDs.STG_DEFECT_ATTR_ODC_TARGET, newWorkItemState.getProjectArea(), workItemCommon, monitor);

		String odcTargetValue = ((Identifier<?>)newWorkItemState.getValue(odcTargetAttr)).getStringIdentifier();

		if(odcTargetValue.equals(REQUIRED_TARGET_VALUE)){
			if(workItemCommonTasks.attributeHasNonNullValue(newWorkItemState, odcContentType, workItemCommon.getAuditableCommon()) == false){
				IAdvisorInfo info= collector.createProblemInfo(odcContentType.getDisplayName() + " required", odcContentType.getDisplayName() + " required when " + odcTargetAttr.getDisplayName() + " equals User Documentation." , ID);
				info.setProblemObject(newWorkItemState.getItemHandle());
				collector.addInfo(info);
			}
		}
	}

}
