package com.ibm.stg.rtc.ext.stgdefect.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.STGCommonIDs;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.IWorkItem;

/**
 * This advisor only runs for Change Records and it sets the Parent ID field to be
 * the universal id of the parent work item. 
 * Note that advisors usually should not modify a work item, however this is a special 
 * case because the parent universal id is only set once on create and we need to avoid sending
 * multiple notifications.
 * 
 * @author Kyle Christianson
 *
 */
public class SetParentUniversalIdAdvisor extends AbstractService implements IOperationAdvisor {

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// check to see if the work item is a Change Record, if not return
		if(!(newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.CHANGE_RECORD_ID))){
			return;
		}

		IAttribute parentIdAttr = workItemCommonTasks.getAttribute(STGDefectCommonIDs.CHANGE_RECORD_ATTR_PARENT_UNIVERSAL_ID, newWorkItemState.getProjectArea(), workItemCommon, monitor);

		// check to see if the work item has the parent universal id attribute set
		if(!workItemCommonTasks.attributeHasNonNullValue(newWorkItemState, parentIdAttr, workItemCommon.getAuditableCommon())){
			IWorkItem parent = workItemCommonTasks.getParentWorkItem(saveParameter.getNewReferences(), workItemCommon.getAuditableCommon(), monitor);
			// only set the parent id if a parent exists (a parent should always exist on the CR)
			if(parent != null){
				// get the parent's universal id
				IAttribute univIdAttr = workItemCommonTasks.getAttribute(STGCommonIDs.STG_ATTR_UNIVERSAL_ID, parent.getProjectArea(), workItemCommon, monitor);
				String parentId = (String)parent.getValue(univIdAttr);

				// if the attribute doesn't exist we need to add it
				if(!newWorkItemState.hasAttribute(parentIdAttr)){
					newWorkItemState.addCustomAttribute(parentIdAttr);
				}

				// set the value
				newWorkItemState.setValue(parentIdAttr, parentId);
			}
		}

	}

}
