package com.ibm.stg.rtc.ext.stgdefect.participants;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.RepositoryCommonTasks;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonTasks;
import com.ibm.team.links.common.IReference;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.TeamOperationCanceledException;
import com.ibm.team.process.common.advice.runtime.IOperationParticipant;
import com.ibm.team.process.common.advice.runtime.IParticipantInfoCollector;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemReferences;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;
import com.ibm.team.workitem.service.IWorkItemServer;

/**
 * This follow-up action only runs for STG Defects and removes the 'Duplicate Of' link if one exists and the
 * resolution is not 'Duplicate'.
 * 
 * @author Kyle Christianson
 *
 */
public class RemoveDuplicateLinkParticipant extends AbstractService implements IOperationParticipant {

	private static final String ID = "com.ibm.stg.rtc.ext.stgdefect.participants.RemoveDuplicateLinkParticipant";

	public void run(AdvisableOperation operation, IProcessConfigurationElement participantConfig, IParticipantInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// check to see if this is an STG Defect
		if (!newWorkItemState.getWorkItemType().equals(STGDefectCommonIDs.STG_DEFECT_ID)){
			return;
		}

		IWorkItemServer workItemServer = this.getService(IWorkItemServer.class);

		boolean duplicateResolution = STGDefectCommonTasks.isResolutionDuplicate(newWorkItemState);
		// leaving the duplicateWIType parameter null means that this will be true if any duplicate work item (of any type) exists
		boolean hasDuplicateOfLink = workItemCommonTasks.hasDuplicateOfLink(saveParameter.getNewReferences(), null, workItemServer.getAuditableCommon(), monitor);

		if(duplicateResolution == false && hasDuplicateOfLink == true){
			// the duplicate link should be removed
			// get a fresh working copy
			IWorkItem workItemWC = (IWorkItem)workItemServer.findWorkItemById(newWorkItemState.getId(), IWorkItem.FULL_PROFILE, null).getWorkingCopy();
			IWorkItemReferences references = workItemServer.resolveWorkItemReferences(workItemWC, monitor);
			List<IReference> duplicateOfLinks = references.getReferences(WorkItemEndPoints.DUPLICATE_OF_WORK_ITEM);
			IReference duplicateOfRef = duplicateOfLinks.get(0); // there can only be one duplicate of link
			references.remove(duplicateOfRef);
			try{
				workItemServer.saveWorkItem2(workItemWC, references, null);
			}catch(TeamOperationCanceledException e){
				RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
				IReportInfo info = collector.createInfo("Follow-up action failed", "Could not remove the duplicate link because a precondition failed with the following message: \n" + repoCommon.getFirstErrorDescription(e));
				info.setProblemObject(newWorkItemState.getItemHandle());
				info.setSeverity(IProcessReport.ERROR);
				collector.addInfo(info);
				// Do not suppress the error messages because this would cause an invalid link so we want the entire transaction to roll back
			} 
		}

	}

}
