package com.ibm.stg.rtc.ext.stgdefect.participants;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.RepositoryCommonTasks;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.stgdefect.common.STGDefectCommonIDs;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.TeamOperationCanceledException;
import com.ibm.team.process.common.advice.runtime.IOperationParticipant;
import com.ibm.team.process.common.advice.runtime.IParticipantInfoCollector;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.model.ICategoryHandle;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.service.IWorkItemServer;

public class SetCRCategoryParticipant extends AbstractService implements IOperationParticipant {

	public static final String ID = "com.ibm.stg.stgdefect.participants.SetCRCategoryParticipant";

	public void run(AdvisableOperation operation, IProcessConfigurationElement participantConfig, IParticipantInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState;
		IWorkItem oldWorkItemState;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		} else if(tempObj == null){
			oldWorkItemState = null;
		} else{
			return;
		}

		IWorkItemServer workItemServer = this.getService(IWorkItemServer.class);
		IAuditableCommon auditableCommon = this.getService(IAuditableCommon.class);


		ICategoryHandle parentCategoryHandle = newWorkItemState.getCategory();
		ICategoryHandle oldCategoryHandle = (oldWorkItemState == null) ? null : oldWorkItemState.getCategory();

		if(parentCategoryHandle != null && !parentCategoryHandle.sameItemId(oldCategoryHandle)){
			// the category has been changed so see if the work item has Change Records
			List<IWorkItem> childCRs = workItemCommonTasks.getChildWorkItems(STGDefectCommonIDs.CHANGE_RECORD_ID, saveParameter.getNewReferences(), auditableCommon, monitor);

			// for all CRs, change the category to match the parent
			for(IWorkItem child : childCRs){
				ICategoryHandle childCategory = child.getCategory();
				if(!parentCategoryHandle.sameItemId(childCategory)){
					// need to check that the change record isn't in a different project
					// if it is in a different project, don't update the category because it won't exist
					if(newWorkItemState.getProjectArea().sameItemId(child.getProjectArea())){
						// get a fresh working copy
						IWorkItem childWC = (IWorkItem)workItemServer.findWorkItemById(child.getId(), IWorkItem.FULL_PROFILE, null).getWorkingCopy();
						childWC.setCategory(parentCategoryHandle);
						try{
							workItemServer.saveWorkItem2(childWC, null, null);
						}catch(TeamOperationCanceledException e){
							RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
							IReportInfo info = collector.createInfo("Follow-up action failed", "Could not set category of child CR " + child.getId() + " because a precondition failed with the following message: \n" + repoCommon.getFirstErrorDescription(e));
							info.setProblemObject(child.getItemHandle());
							info.setSeverity(IProcessReport.ERROR);
							collector.addInfo(info);
							// Do not suppress the error messages because this would cause an invalid category so we want the entire transaction to roll back
						} 
					}
				}
			}
		}
	}
}
