package com.ibm.stg.rtc.ext;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URL;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;
import org.osgi.framework.Version;

import com.ibm.team.repository.service.TeamRawService;

public class STGVersionService extends TeamRawService implements ISTGVersionService{

	private static final String BUNDLE_SYMBOLIC_NAME = "com.ibm.stg.rtc.ext";
	private static final String VERSION_FILE_PATH = "version.txt";

	@Override
	public void perform_GET(String uri, HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		response.setContentType("text/plain");
		PrintWriter writer = response.getWriter();

		writer.println("Bundle: " + BUNDLE_SYMBOLIC_NAME + "\nBundle version: " + getBundleVersion(BUNDLE_SYMBOLIC_NAME));

		String versionFileContents = getPluginFileContentsAsString(BUNDLE_SYMBOLIC_NAME, VERSION_FILE_PATH);

		writer.println("Version file contents: \n\n" + versionFileContents);

		writer.close();
	}

	private String getBundleVersion(String symbolicName){
		Bundle bundle = Platform.getBundle(symbolicName);
		Version version = bundle.getVersion();
		return version.toString();
	}


	/*
	 * Using the example below, I've created this method to return the contents of a
	 * bundle file as a string.
	 */
	private String getPluginFileContentsAsString(String symbolicName, String path) throws IOException {
		InputStream is = null;
		StringBuilder sb = new StringBuilder();
		try {
			Bundle b = Platform.getBundle(symbolicName);

			URL url = b.getEntry(path);
			if (url != null) {
				is = url.openStream();
				InputStreamReader reader = new InputStreamReader(is);
				char[] buffer = new char[4096];
				int len = 0;
				while ((len = reader.read(buffer, 0, buffer.length)) != -1) {
					sb.append(buffer, 0, len);
				}
			}
		}
		finally {
			if (is != null) {try {is.close();}catch(IOException e){}}
		}
		return sb.toString();
	}

	/* 
	 * this is an example on how to get a resource file from a bundle
	 * see https://jazz.net/wiki/bin/view/Main/JazzServerProvisioningInternals
	 *
	private static File copyPluginFile(String symbolicName, String path) throws IOException {
		File f = null;
		InputStream is = null;
		OutputStream os = null;
		try {
			Bundle b = Platform.getBundle(symbolicName);

			URL url = b.getEntry(path);
			if (url != null) {
				is = url.openStream();
				f = b.getBundleContext().getDataFile(path);
				f.getParentFile().mkdirs();
				os = new BufferedOutputStream (new FileOutputStream(f));
				byte[] buffer = new byte[4096];
				int len = 0;
				while ((len = is.read(buffer, 0, 4096)) != -1) {
					os.write(buffer, 0, len);
				}
			}
		}
		finally {
			if (is != null) {try {is.close();}catch(IOException e){}}
			if (os != null) {try {os.close();}catch(IOException e){}}
		}
		return f;
	} */

}
