package com.ibm.stg.rtc.ext.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.ConfigurationHelper;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.ICategory;
import com.ibm.team.workitem.common.model.ICategoryHandle;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemType;

public class PreventCategoryUsageAdvisor extends AbstractService implements IOperationAdvisor {
	public static final String ID = "com.ibm.stg.rtc.ext.advisors.PreventCategoryUsageAdvisor";
		
	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration,
			IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		ISaveParameter saveParameter = null;
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		IAuditableCommon auditableClient = this.getService(IAuditableCommon.class);
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();		
		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}
		
		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		} else if(tempObj == null){
			oldWorkItemState = null;
		} else{
			return;
		}


		//need to switch to using common code
		// check to see if the work item type has prevent usage configured for the category set on the work item, if not return
		String workItemTypeId = newWorkItemState.getWorkItemType();			
		
		ICategoryHandle newCategory = newWorkItemState.getCategory();
		ICategoryHandle oldCategory = (oldWorkItemState == null) ? null : oldWorkItemState.getCategory();
		boolean categoryChanging = isCategoryChanging(saveParameter.isCreation(), newCategory, oldCategory);
		
		//category didn't change so don't check if the category is valid or not
		if (!categoryChanging) {
			return;
		}
		
		ICategory category = auditableClient.resolveAuditable(newCategory, ICategory.FULL_PROFILE, null);
		
		//if the category cannot be used, error
		if (!ConfigurationHelper.isCategoryUsable(category, workItemTypeId, workItemCommon)) {		
			IWorkItemType workItemType = workItemCommon.findWorkItemType(newWorkItemState.getProjectArea(), newWorkItemState.getWorkItemType(), monitor);
			String typeName;
			if(workItemType != null) {
				typeName = workItemType.getDisplayName();
			}
			else {
				typeName = newWorkItemState.getState2().getStringIdentifier();
			}						
			
			IAdvisorInfo info = collector.createProblemInfo("Cannot use Category", "Category '" + category.getName()
					+ "' cannot be used on " + typeName
					+ " work items.  Please pick a different value for Filed Against.", ID);
			info.setProblemObject(newWorkItemState.getItemHandle());
			collector.addInfo(info);
			return;
		}
	}
	
	/**
	 * Returns true iff the category is changing. A new work item means it is changing.
	 * @param creation
	 * @param newWorkItemState
	 * @param oldWorkItemState
	 * @return
	 */
	private boolean isCategoryChanging(boolean creation, ICategoryHandle newCategory, ICategoryHandle oldCategory) {
		if(creation == true){
			return true;
		}else if (newCategory == null){
			return oldCategory == null;
		}else{
			return !newCategory.sameItemId(oldCategory);
		}
	}
}
