package com.ibm.stg.rtc.ext.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.IProcessItem;
import com.ibm.team.process.common.IProjectArea;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.process.internal.service.IProcessItemSaveOperation;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.repository.service.IRepositoryItemService;

/**
 * This advisor prevents project rename.
 * 
 * @author Kyle Christianson
 *
 */
@SuppressWarnings("restriction")
public class PreventProjectRenameAdvisor extends AbstractService implements IOperationAdvisor {

	private static final String ID = "com.ibm.stg.rtc.ext.advisors.PreventProjectRenameAdvisor";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {

		IProcessItemSaveOperation processSaveOperation;

		if(operation instanceof IProcessItemSaveOperation){
			processSaveOperation = (IProcessItemSaveOperation)operation;
		}else{
			return;
		}

		IRepositoryItemService repositoryItemService = (IRepositoryItemService)this.getService(IRepositoryItemService.class);

		IProcessItem[] items = processSaveOperation.getItems();
		for(IProcessItem item : items){
			if(item.getProcessItemType() == IProcessItem.PROJECT_AREA){
				IProjectArea newProjectState = (IProjectArea)item;
				// fetch the old project state from the repository
				IProjectArea oldProjectState = (IProjectArea) repositoryItemService.fetchItem(newProjectState, IRepositoryItemService.COMPLETE);

				String newProjectName = newProjectState.getName();
				String oldProjectName = oldProjectState.getName();

				if(oldProjectName.equals(newProjectName) == false){
					// error because changed name
					IReportInfo info = collector.createProblemInfo("Permission Denied", "You don't have permission to rename the project area", ID);
					info.setProblemObject(newProjectState.getItemHandle());
					collector.addInfo(info);
				}
			}

		}
	}
}