package com.ibm.stg.rtc.ext.advisors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.PermissionsHelper;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.process.service.IProcessServerService;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.service.IPermissionService;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.repository.service.IRepositoryItemService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.model.ISubscriptions;
import com.ibm.team.workitem.common.model.IWorkItem;

public class PreventSubscriberRemovalAdvisor extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.stgdefect.advisors.PreventSubscriberRemovalAdvisor";

	private static final String TAG_WORKITEM_TYPE = "workItemType";
	private static final String ATTR_ID = "id";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		IRepositoryItemService repositoryItemService = this.getService(IRepositoryItemService.class);
		IPermissionService permissionService = this.getService(IPermissionService.class);
		IProcessServerService processServerService = this.getService(IProcessServerService.class);

		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		IContributor authenticatedContributor = (IContributor)repositoryItemService.fetchItem(this.getAuthenticatedContributor(), IRepositoryItemService.COMPLETE);
		if (PermissionsHelper.canUserDeleteSubscriber(saveParameter, authenticatedContributor, processServerService, permissionService)) {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		// check to see if this work item type is configured
		if (!getConfiguredWITypes(advisorConfiguration).contains(newWorkItemState.getWorkItemType())) {
			return;
		}

		ISubscriptions newSubscriptions = newWorkItemState.getSubscriptions();
		ISubscriptions oldSubscriptions = oldWorkItemState.getSubscriptions();
		IContributorHandle[] oldContributorHandles = oldSubscriptions.getContents();
		IContributorHandle[] newContributorHandles = newSubscriptions.getContents();

		IContributor oldTempContributor = null;
		IContributor newTempContributor = null;
		IContributor loggedInContributor = (IContributor) repositoryItemService.fetchItem(this.getAuthenticatedContributor(), IRepositoryItemService.COMPLETE);
		boolean contributorInList = false;
		for (int oc = 0; oc < oldContributorHandles.length; oc++) {
			oldTempContributor = (IContributor) repositoryItemService.fetchItem(oldContributorHandles[oc], IRepositoryItemService.COMPLETE);
			if (! oldTempContributor.sameItemId(loggedInContributor)) {
				for (int nc = 0; nc < newContributorHandles.length; nc++) {
					newTempContributor = (IContributor) repositoryItemService.fetchItem(newContributorHandles[nc], IRepositoryItemService.COMPLETE);
					if (oldTempContributor.sameItemId(newTempContributor)) {
						contributorInList = true;
						break;
					}
				}
				if (!contributorInList) {
					IAdvisorInfo info = collector.createProblemInfo("Can only remove yourself from subscribers list.", "You can only remove your own user ID from the list of subscribers.", ID);
					info.setProblemObject(newWorkItemState.getItemHandle());
					collector.addInfo(info);
					return;
				}
				contributorInList = false;
			}
		}
	}

	private List<String> getConfiguredWITypes(IProcessConfigurationElement advisorConfiguration){
		List<String> workItemTypes = new ArrayList<String>();

		IProcessConfigurationElement[] configurationElements = advisorConfiguration.getChildren();

		// loop through all the child xml elements for this precondition
		for (IProcessConfigurationElement configurationElement : configurationElements) {
			// if the element is a workItemType element, add the id to the list
			if (configurationElement.getName().equals(TAG_WORKITEM_TYPE)) {
				workItemTypes.add(configurationElement.getAttribute(ATTR_ID));
			}
		}
		return workItemTypes;
	}

}
