package com.ibm.stg.rtc.ext.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.process.common.IDevelopmentLine;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.IProcessItem;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.process.internal.service.IProcessItemSaveOperation;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.repository.service.IRepositoryItemService;

/**
 * This advisor prevents Timeline Name and Identifier changes in RTC.
 * @author Kyle Christianson
 *
 */
@SuppressWarnings("restriction")
public class PreventTimelineEditAdvisor extends AbstractService implements IOperationAdvisor {

	private static final String ID = "com.ibm.stg.rtc.ext.advisors.PreventTimelineEditAdvisor";

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {

		IProcessItemSaveOperation processSaveOperation;

		if(operation instanceof IProcessItemSaveOperation){
			processSaveOperation = (IProcessItemSaveOperation)operation;
		}else{
			return;
		}

		IRepositoryItemService repositoryItemService = (IRepositoryItemService)this.getService(IRepositoryItemService.class);

		IProcessItem[] items = processSaveOperation.getItems();
		for(IProcessItem item : items){
			if(item.getProcessItemType() == IProcessItem.DEVELOPMENT_LINE){
				if(item.isNewItem() == true){
					// error because user is creating a new timeline
					//IReportInfo info = collector.createProblemInfo("Permission Denied", "Timelines must be created in Metadata Plus as Internal Releases", ID);
					IReportInfo info = collector.createProblemInfo("Permission Denied", "To create timelines you must submit an ITHelp Defect. Please see https://stgdoc.rchland.ibm.com/STGC/RTC_FAQ#whynotcreate for more information.", ID);
					info.setProblemObject(item.getItemHandle());
					collector.addInfo(info);
					return;
				}else{
					// first get the current working copy of the development line
					IDevelopmentLine newDevLineState = (IDevelopmentLine)item;
					// now get copy of the development line stored in the repository
					IDevelopmentLine curDevLineState = (IDevelopmentLine)repositoryItemService.fetchItem(newDevLineState, IRepositoryItemService.COMPLETE);
					if(newDevLineState.getName().equals(curDevLineState.getName()) == false){
						// error because changed name
						IReportInfo info = collector.createProblemInfo("Permission Denied", "To rename timelines you must submit an ITHelp Defect. Please see https://stgdoc.rchland.ibm.com/STGC/RTC_FAQ#whynotcreate for more information.", ID);
						info.setProblemObject(newDevLineState.getItemHandle());
						collector.addInfo(info);
					}
					if(newDevLineState.getId().equals(curDevLineState.getId()) == false){
						// error because changed identifier
						IReportInfo info = collector.createProblemInfo("Permission Denied", "Timeline identifiers cannot be modified.", ID);
						info.setProblemObject(newDevLineState.getItemHandle());
						collector.addInfo(info);
					}
				}
			}
		}
	}
}
