package com.ibm.stg.rtc.ext.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.STGCommonIDs;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.IWorkItem;

/**
 * This operation advisor checks to see if the blocking field is being set. If so,
 * a new comment is required or this precondition will fail.
 * 
 * @author christik
 *
 */
public class RequireBlockingCommentAdvisor extends AbstractService implements IOperationAdvisor {

	public static final String ID = "com.ibm.stg.stgdefect.advisors.RequireCommentAdvisor";


	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		ISaveParameter saveParameter = null;
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState = null;



		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		} else if(tempObj == null){
			oldWorkItemState = null;
		} else{
			return;
		}


		IAttribute blockingAttribute = workItemCommonTasks.getAttribute(STGCommonIDs.STG_ATTR_BLOCKING, newWorkItemState.getProjectArea(), workItemCommon, monitor);

		if(newWorkItemState.hasAttribute(blockingAttribute) == false){
			// the work item does not have the blocking attribute
			return;
		}


		boolean oldBlockingValue = false;
		if(oldWorkItemState != null && oldWorkItemState.hasAttribute(blockingAttribute)){
			oldBlockingValue = (Boolean)oldWorkItemState.getValue(blockingAttribute);
		}

		boolean newBlockingValue = (Boolean)newWorkItemState.getValue(blockingAttribute);

		if(oldBlockingValue == false && newBlockingValue == true){
			// a comment is required
			int oldNumComments = 0;
			if(oldWorkItemState != null){
				oldNumComments = oldWorkItemState.getComments().getContents().length;
			}

			int newNumComments = newWorkItemState.getComments().getContents().length;

			if(newNumComments == oldNumComments){
				IAdvisorInfo info= collector.createProblemInfo("Comment Required", "A new comment is required to describe what is blocked because you have enabled the blocking flag", ID);
				info.setProblemObject(newWorkItemState.getItemHandle());
				collector.addInfo(info);
			}
		}
	}

}
