package com.ibm.stg.rtc.ext.advisors;

import static com.ibm.team.scm.common.internal.process.AdvisorDetailConstants.ATTR_CONTEXT_TYPE;
import static com.ibm.team.scm.common.internal.process.AdvisorDetailConstants.ATTR_CONTEXT_UUID;
import static com.ibm.team.scm.common.internal.process.AdvisorDetailConstants.ATTR_REPO;
import static com.ibm.team.scm.common.internal.process.AdvisorDetailConstants.ATTR_UUIDS;
import static com.ibm.team.scm.common.internal.process.AdvisorDetailConstants.VAL_CONTEXT_TYPE_WORKSPACE;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import javax.xml.transform.TransformerException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ibm.team.filesystem.common.workitems.internal.process.RequireWorkItemAdvisor;
import com.ibm.team.filesystem.common.workitems.internal.process.RequireWorkItemProblemObject;
import com.ibm.team.filesystem.service.internal.Messages;
import com.ibm.team.links.common.ILink;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.IProjectArea;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.process.internal.common.model.AbstractModel;
import com.ibm.team.repository.common.IAuditable;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.repository.common.IRepositoryRoot;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.service.IRepositoryItemService;
import com.ibm.team.scm.common.IChangeSet;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IWorkspaceHandle;
import com.ibm.team.scm.common.internal.dto.HistoryProvider;
import com.ibm.team.scm.common.internal.dto.WorkspaceComponentPair;
import com.ibm.team.scm.common.internal.process.AdvisorDetailConstants;
import com.ibm.team.scm.common.internal.process.UnifiedProcessDescriptionUtil;
import com.ibm.team.scm.common.links.ChangeSetLinks;
import com.ibm.team.scm.common.links.ILinkConstants;
import com.ibm.team.scm.common.process.IChangeHistoryAddOperandDelta;
import com.ibm.team.scm.common.process.IScmDeltaSource;
import com.ibm.team.scm.common.providers.ItemProvider;
import com.ibm.team.scm.common.providers.ProviderFactory;
import com.ibm.team.scm.service.internal.AbstractScmService;
import com.ibm.team.scm.service.internal.process.ClientVersionHelper;
import com.ibm.team.scm.service.internal.process.ClientVersionHelper.RemoteCapabilities;
import com.ibm.team.scm.service.internal.utils.ServerProviderFactory;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;



/**
 * Operation advisor which validates that all change sets have an associated
 * work item and/or comment.
 * 
 * christik - This is a copy of com.ibm.team.filesystem.service.internal.ServerRequireWorkItemAdvisor
 * except for one distinct change: A remote change request is treated as a work item rather than a comment.
 * 
 * Note: This precondition does not honor the required owner or target options for change requests.
 */
@SuppressWarnings({ "restriction", "rawtypes", "unchecked" })
public class RequireChangeSetLinkAdvisor extends AbstractScmService implements IOperationAdvisor {


	private static String ELEMENT_REQUIRES = "requires"; //$NON-NLS-1$
	private static String ATTR_VALUE = "value"; //$NON-NLS-1$

	public static String VAL_WORKITEM = "workitem"; //$NON-NLS-1$
	public static String VAL_COMMENT = "comment"; //$NON-NLS-1$
	public static String VAL_WORKITEM_OR_COMMENT = "workitem-or-comment"; //$NON-NLS-1$
	public static String VAL_WORKITEM_AND_COMMENT = "workitem-and-comment"; //$NON-NLS-1$

	private static String ELEMENT_WORKITEM = "workitem"; //$NON-NLS-1$
	private static String ATTR_TARGET = "target"; //$NON-NLS-1$
	private static String ATTR_OWNER = "owner"; //$NON-NLS-1$

	private static String VAL_REQUIRED = "required"; //$NON-NLS-1$
	private static String VAL_OPTIONAL = "optional"; //$NON-NLS-1$
	private static String VAL_CURRENT_ITERATION = "current-iteration"; //$NON-NLS-1$
	private static String VAL_CURRENT_CONTRIBUTOR = "current-contributor"; //$NON-NLS-1$

	public static final String ID_REQUIRE_WORK_ITEM_PROBLEM = "com.ibm.team.process.definitions.requireWorkItemProblem"; //$NON-NLS-1$

	// Problem identifier indicating that there is a problem with the configuration of this advisor
	public static final String ID_CONFIGURATION_PROBLEM = "com.ibm.team.process.definitions.requireWorkItem.configurationProblem"; //$NON-NLS-1$

	/*
	 * By default, the advisor requires a workitem. Configuration of the
	 * 'requires' attribute is optional.
	 */
	private String fRequires = VAL_WORKITEM;
	private String fTarget = VAL_OPTIONAL;
	private String fOwner = VAL_OPTIONAL;

	private IRepositoryRoot root;

	// Generic access to common services available on both the client and the server.
	private ProviderFactory provider;
	private ItemProvider items;

	/* christik - remove the constructor and set the variables when run() is called
	public ServerRequireWorkItemAdvisor(IRepositoryRoot root, ProviderFactory provider) {
		this.root = root;
		this.provider = provider;

		items = provider.getItemProvider();
	}*/

	@SuppressWarnings("serial")
	public static class AddProblemFailedException extends Exception {
	}

	/**
	 * Initializes this advisor from the given configuration.
	 */
	private void initialize(IProcessConfigurationElement advisorConfiguration) {
		IProcessConfigurationElement[] children = advisorConfiguration.getChildren();
		for (int i = 0; i < children.length; i++) {
			IProcessConfigurationElement child = children[i];
			String childName = child.getName();
			if (ELEMENT_REQUIRES.equals(childName)) {
				fRequires = child.getAttribute(ATTR_VALUE);
				IProcessConfigurationElement[] grandChildren = child.getChildren();
				for (int j = 0; j < grandChildren.length; j++) {
					IProcessConfigurationElement grandChild = grandChildren[j];
					String grandChildName = grandChild.getName();
					if (ELEMENT_WORKITEM.equals(grandChildName)) {
						fTarget = grandChild.getAttribute(ATTR_TARGET);
						fOwner = grandChild.getAttribute(ATTR_OWNER);
					}
				}
			}
		}
	}



	/*
	 * @see com.ibm.team.process.common.advice.runtime.IOperationAdvisor#run(com.ibm.team.process.common.advice.AdvisableOperation, com.ibm.team.process.common.IProcessConfigurationElement, com.ibm.team.process.common.advice.IAdvisorInfoCollector, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor progress) throws TeamRepositoryException {
		Object opData = operation.getOperationData();
		if (! (opData instanceof IScmDeltaSource)) {
			// Process should protect us from this. If we ever get this, 
			// we're either producing the wrong kind of data objects, or someone 
			// has messed with our plugin.xml.
			throw new TeamRepositoryException("I only consume deltas"); //$NON-NLS-1$
		}

		// christik - set up the member variables and here instead of the constructor similar
		// to com.ibm.team.filesystem.service.internal.ServerRequireWorkItemAdvisorRunner
		IRepositoryItemService itemService = getService(IRepositoryItemService.class);
		this.root = itemService.getRepositoryRoot();
		this.provider = new ServerProviderFactory(this, itemService);
		this.items = this.provider.getItemProvider();

		IScmDeltaSource src = (IScmDeltaSource) opData;

		SubMonitor monitor = SubMonitor.convert(progress, Messages.getString("RequireWorkItemAdvisor.PROGRESS"), 10); //$NON-NLS-1$
		try {
			initialize(advisorConfiguration);
			IProcessArea processArea = operation.getProcessArea();
			IProjectAreaHandle handle = processArea.getProjectArea();
			IProjectArea projectArea = (IProjectArea) items.fetchItem(handle, monitor.newChild(1));

			addProblems(src, processArea, projectArea, collector, monitor.newChild(9));
		} catch (TeamRepositoryException e) {
			collector.addInfo(collector.createExceptionInfo(Messages.getString("RequireWorkItemAdvisor.ERROR_MSG"), e)); //$NON-NLS-1$
		} finally {
			monitor.done();
		}
	}

	private void addProblems(IScmDeltaSource src, IProcessArea processArea, final IProjectArea projectArea, IAdvisorInfoCollector collector, SubMonitor monitor) throws TeamRepositoryException {
		try {
			if (!isValidRequiresValue(fRequires)) {
				collector.addInfo(collector.createProblemInfo(Messages.getString("RequireWorkItemAdvisor.INTERNAL_CONFIGURATION_ERROR"), NLS.bind(Messages.getString("RequireWorkItemAdvisor.3"), new Object[] { VAL_WORKITEM, VAL_COMMENT, VAL_WORKITEM_OR_COMMENT, VAL_WORKITEM_AND_COMMENT}), ID_CONFIGURATION_PROBLEM)); //$NON-NLS-1$ //$NON-NLS-2$
				monitor.done();
				return;
			}
			if (!isValidTargetValue(fTarget)) {
				collector.addInfo(collector.createProblemInfo(Messages.getString("RequireWorkItemAdvisor.INTERNAL_CONFIGURATION_ERROR"), NLS.bind(Messages.getString("RequireWorkItemAdvisor.5"), new Object[] { VAL_OPTIONAL, VAL_REQUIRED, VAL_CURRENT_ITERATION }), ID_CONFIGURATION_PROBLEM)); //$NON-NLS-1$ //$NON-NLS-2$
			}
			if (!isValidOwnerValue(fOwner)) {
				collector.addInfo(collector.createProblemInfo(Messages.getString("RequireWorkItemAdvisor.INTERNAL_CONFIGURATION_ERROR"), NLS.bind(Messages.getString("RequireWorkItemAdvisor.7"), new Object[] { VAL_OPTIONAL, VAL_REQUIRED, VAL_CURRENT_CONTRIBUTOR }), ID_CONFIGURATION_PROBLEM)); //$NON-NLS-1$ //$NON-NLS-2$
			}

			for (IChangeHistoryAddOperandDelta delta : src.getDeltas(IChangeHistoryAddOperandDelta.class)) {
				monitor.setWorkRemaining(10);

				SubMonitor loopMonitor = monitor.newChild(1);
				loopMonitor.setWorkRemaining(delta.getComponents().size() * 2);

				for (IComponentHandle comp : delta.getComponents()) {
					List<IChangeSetHandle> changes = new LinkedList<IChangeSetHandle>();

					for (IChangeSetHandle cs : delta.getAdditionalBaselineChangeSets(comp)) {
						changes.add(cs);
					}

					for (IChangeSetHandle cs : delta.getAdditionalChangeSets(comp)) {
						changes.add(cs);
					}


					Map<IChangeSet, List<IAuditable>> links = ChangeSetLinks.resolveLinks(provider, changes, ILinkConstants.CHANGESET_WORKITEM_LINKTYPE_ID, loopMonitor.newChild(1));


					loopMonitor.setWorkRemaining(links.size());

					for (IChangeSet changeSet : links.keySet()) {
						String comment = null;

						List workItemHandles = new ArrayList();
						List<ILink> oslcLinks = ChangeSetLinks.findLinks(provider, changeSet, new String[] { ILinkConstants.CHANGESET_OSLC_CM_CHANGEREQUEST_LINKTYPE_ID }, monitor);

						if (changeSet != null) {
							comment = changeSet.getComment();
							List<IAuditable> linkTargets = links.get(changeSet);
							for (IAuditable referencedItem : linkTargets) {

								if(referencedItem instanceof IWorkItemHandle) {
									workItemHandles.add(referencedItem);
								}
							}
						}

						try {
							addProblem(delta, comment, !oslcLinks.isEmpty(), workItemHandles, changeSet, collector, projectArea, processArea, loopMonitor.newChild(1));
						} catch (AddProblemFailedException e) {
							// The add-problem failed, but the problem has already been
							// reported via the collector. Keep running. 
						}
					}
				}
			}
		} finally {
			monitor.done();
		}
	}

	private RequireWorkItemProblemObject createProblemObject(IChangeHistoryAddOperandDelta delta, IChangeSet changeSet, List workItems, IProjectArea projectArea, boolean ownerMustBeUser, boolean isNeedTarget, boolean isNeedOwner, boolean wantsWorkItem, boolean wantsComment) throws TeamRepositoryException {
		HistoryProvider history = (HistoryProvider)delta.getSource(changeSet.getComponent());

		IWorkspaceHandle wsHandle = null;
		if (history != null) {
			WorkspaceComponentPair pair = history.getWorkspaceComponentPair();
			if (pair != null) {
				wsHandle = pair.getWorkspace();
			}
		}

		return new RequireWorkItemProblemObject(root.getItemId(), wsHandle, 
				changeSet.getComponent(), changeSet, workItems, projectArea, fRequires, ownerMustBeUser, isNeedTarget, 
				isNeedOwner, wantsWorkItem, wantsComment);
	}

	private IAdvisorInfo createProblem(String summary, String description, Document doc, IAdvisorInfoCollector collector) throws TeamRepositoryException {
		// christik - use the ID_REQUIRE_WORK_ITEM_PROBLEM id so the client AssociateWorkItemResolutionDelegate can suggest solutions (quick-fixes)
		// to the problem in the TeamAdvisor view. However, the TeamAdvisor will not look exactly the same because we have not registered this 
		// advisor with the com.ibm.team.process.ide.ui.detailProviderFactories extension point. This would require a client side plug-in.
		IAdvisorInfo problem = collector.createProblemInfo(summary, description, ID_REQUIRE_WORK_ITEM_PROBLEM);

		Exception e = null;
		try {
			String serialized = AbstractModel.serializeDocument(doc);

			problem.setData(serialized);
		} catch (TransformerException exception) {
			e = exception;
		} catch (IOException exception) {
			e = exception;
		}

		if (e != null) {
			problem = collector.createExceptionInfo(Messages.getString("RequireWorkItemAdvisor.33"), e); //$NON-NLS-1$
		}

		return problem;
	}

	private boolean isValidRequiresValue(String requiresValue) {
		return VAL_WORKITEM.equals(requiresValue) || VAL_COMMENT.equals(requiresValue) || VAL_WORKITEM_OR_COMMENT.equals(requiresValue) ||VAL_WORKITEM_AND_COMMENT.equals(requiresValue);
	}

	private boolean isValidTargetValue(String targetValue) {
		return VAL_OPTIONAL.equals(targetValue) || VAL_REQUIRED.equals(targetValue) || VAL_CURRENT_ITERATION.equals(targetValue);
	}

	private boolean isValidOwnerValue(String ownerValue) {
		return VAL_OPTIONAL.equals(ownerValue) || VAL_REQUIRED.equals(ownerValue) || VAL_CURRENT_CONTRIBUTOR.equals(ownerValue);
	}


	private void addProblem(IChangeHistoryAddOperandDelta delta, String comment, boolean hasChangeRequest, List workItems, IChangeSet changeSet, IAdvisorInfoCollector collector, IProjectArea projectArea, IProcessArea processArea, IProgressMonitor monitor) throws TeamRepositoryException, AddProblemFailedException {
		monitor.beginTask(Messages.getString("RequireWorkItemAdvisor.8"), 10); //$NON-NLS-1$

		RemoteCapabilities clientVersion = ClientVersionHelper.getClientVersion();

		try {
			// First check if the change set is missing a required workitem or comment
			final boolean hasWorkitems = workItems != null && !workItems.isEmpty();
			{
				boolean hasComment = comment != null && comment.trim().length() > 0;
				// christik - removing the hasCommentOrChangeRequest and adding hasWorkitemsOrChangeRequest
				//boolean hasCommentOrChangeRequest = hasChangeRequest || hasComment;
				boolean hasWorkitemsOrChangeRequest = hasWorkitems || hasChangeRequest;

				RequireWorkItemProblemObject problem = null;
				String summary = null;//Plain text heading for problem
				String description = null; // Plain text problem description
				// christik - change hasWorkitems to hasWorkitemsOrChangeRequest, and hasCommentOrChangeRequest to hasComment
				if (VAL_WORKITEM_AND_COMMENT.equals(fRequires) && !hasWorkitemsOrChangeRequest && !hasComment) {
					problem = createProblemObject(delta, changeSet, workItems, projectArea, false, false, false, true, true);
					summary = Messages.getString("RequireWorkItemAdvisor.9"); //$NON-NLS-1$
					description = Messages.getString("RequireWorkItemAdvisor.MISSING_WORK_ITEM_AND_COMMENT"); //$NON-NLS-1$
					// christik - change hasWorkitems to hasWorkitemsOrChangeRequest
				} else if ((VAL_WORKITEM_AND_COMMENT.equals(fRequires) || VAL_WORKITEM.equals(fRequires)) && !hasWorkitemsOrChangeRequest) {
					problem = createProblemObject(delta, changeSet, workItems, projectArea, false, false, false, true, false);
					summary = Messages.getString("RequireWorkItemAdvisor.11"); //$NON-NLS-1$
					description = Messages.getString("RequireWorkItemAdvisor.MISSING_WORK_ITEM"); //$NON-NLS-1$
					// christik - change hasCommentOrChangeRequest to hasComment
				} else if ((VAL_WORKITEM_AND_COMMENT.equals(fRequires) || VAL_COMMENT.equals(fRequires)) && !hasComment) {
					problem = createProblemObject(delta, changeSet, workItems, projectArea, false, false, false, false, true);
					summary = Messages.getString("RequireWorkItemAdvisor.13"); //$NON-NLS-1$
					description = Messages.getString("RequireWorkItemAdvisor.MISSING_CHANGE_REQUEST_OR_COMMENT"); //$NON-NLS-1$
					// christik - change hasWorkitems to hasWorkitemsOrChangeRequest, and hasCommentOrChangeRequest to hasComment
				} else if (VAL_WORKITEM_OR_COMMENT.equals(fRequires) && !hasWorkitemsOrChangeRequest && !hasComment) {
					problem = createProblemObject(delta, changeSet, workItems, projectArea, false, false, false, true, true);
					summary = Messages.getString("RequireWorkItemAdvisor.15"); //$NON-NLS-1$
					description = Messages.getString("RequireWorkItemAdvisor.MISSING_CHANGE_REQUEST_WORKITEM_OR_COMMENT"); //$NON-NLS-1$

				}

				if (problem != null) {
					Document doc = UnifiedProcessDescriptionUtil.createXmlDoc();

					RequireWorkItemProblemObject.deflateInto(problem, clientVersion.consumesAML(), doc);

					collector.addInfo(createProblem(summary, description, doc, collector));
				}
			}

			// If the change set has a workitem, then make sure it meets requirements 
			IReportInfo info = null;
			if (hasWorkitems) {
				workItems = items.fetchItems(workItems, new SubProgressMonitor(monitor, 4));
				for (ListIterator iterator = workItems.listIterator(); iterator.hasNext();) {
					if (iterator.next() == null) {
						collector.addInfo(collector.createExceptionInfo(Messages.getString("RequireWorkItemAdvisor.ERROR_MSG"), new TeamRepositoryException(Messages.getString("RequireWorkItemAdvisor.18")))); //$NON-NLS-1$ //$NON-NLS-2$
						iterator.remove();
					}
				}

				if (!checkTarget(workItems, processArea, new SubProgressMonitor(monitor, 3))) {
					final String summary;
					final String desc;
					if (VAL_REQUIRED.equals(fTarget)) {
						if (workItems.size() == 1) {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_WI_NEEDS_TARGET"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.22"); //$NON-NLS-1$
						} else {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_WI_NEEDS_TARGET"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.23"); //$NON-NLS-1$
						}
					} else {
						if (workItems.size() == 1) {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_WI_NEEDS_CURRENT_ITER"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.24"); //$NON-NLS-1$

						} else {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_WIS_NEEDS_CURRENT_ITER"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.25"); //$NON-NLS-1$
						}
					}

					RequireWorkItemProblemObject problem = createProblemObject(delta, changeSet, workItems, projectArea, false, true, false, false, false);
					Document doc = UnifiedProcessDescriptionUtil.createXmlDoc();

					RequireWorkItemProblemObject.deflateInto(problem, clientVersion.consumesAML(), doc);

					info = createProblem(summary, desc.toString(), doc, collector);
					collector.addInfo(info);
				}

				if (!checkOwner(workItems, new SubProgressMonitor(monitor, 3))) {
					final String summary;
					final String desc;
					if (VAL_REQUIRED.equals(fOwner)) {
						if (workItems.size() == 1) {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_WI_NEEDS_OWNER"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.28"); //$NON-NLS-1$
						} else {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_WIS_NEED_OWNER"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.29"); //$NON-NLS-1$
						}
					} else {
						if (workItems.size() == 1) {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_OWNER_IS_NOT_YOU"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.30"); //$NON-NLS-1$
						} else {
							summary = Messages.getString("RequireWorkItemAdvisor.SUMMARY_OWNERS_ARE_NOT_YOU"); //$NON-NLS-1$
							desc = Messages.getString("RequireWorkItemAdvisor.31"); //$NON-NLS-1$
						}
					}


					RequireWorkItemProblemObject problem = createProblemObject(delta, changeSet, workItems, projectArea, true, false, true, false, false);	            	
					Document doc = UnifiedProcessDescriptionUtil.createXmlDoc();
					RequireWorkItemProblemObject.deflateInto(problem, clientVersion.consumesAML(), doc);

					info = createProblem(summary, desc.toString(), doc, collector);
					collector.addInfo(info);
				}
			}
		} finally {
			monitor.done();
		}
	}

	public static void saveChangesetErrorDescriptionMarkup(UUID repoRoot, IChangeHistoryAddOperandDelta delta, IChangeSet changeSet, Document doc, String xml, String replText, IAdvisorInfoCollector collector) throws AddProblemFailedException {
		Element root = doc.createElement(AdvisorDetailConstants.EL_DESCRIPTION);
		if (doc.getFirstChild() == null) {
			doc.appendChild(root);
		} else {
			doc.getFirstChild().appendChild(root);
		}

		Element para = doc.createElement(AdvisorDetailConstants.EL_PARAGRAPH);
		root.appendChild(para);

		Element link = doc.createElement(AdvisorDetailConstants.EL_CHANGESET_LINK);
		link.setTextContent(Messages.getString("RequireWorkItemAdvisor.CHANGE_SET")); //$NON-NLS-1$
		link.setAttribute(ATTR_UUIDS, changeSet.getItemId().getUuidValue());
		link.setAttribute(ATTR_REPO, repoRoot.getUuidValue());


		link.setAttribute(ATTR_CONTEXT_TYPE, VAL_CONTEXT_TYPE_WORKSPACE);
		try {
			HistoryProvider source = (HistoryProvider)delta.getSource(changeSet.getComponent());
			if (source == null) {
				link.setAttribute(ATTR_CONTEXT_UUID, ""); //$NON-NLS-1$
			} else {
				link.setAttribute(ATTR_CONTEXT_UUID, source.getWorkspaceComponentPair().getWorkspace().getItemId().getUuidValue());
			}
		} catch (TeamRepositoryException e) {
			IReportInfo exceptionInfo = collector.createExceptionInfo(Messages.getString("RequireWorkItemAdvisor.COULD_NOT_CREATE_DESCRIPTION_DOC"), e); //$NON-NLS-1$
			exceptionInfo.setSeverity(IProcessReport.ERROR);
			collector.addInfo(exceptionInfo);
			throw new AddProblemFailedException();
		}

		link.setAttribute(AdvisorDetailConstants.ATTR_COMPONENT_UUID, changeSet.getComponent().getItemId().getUuidValue());


		String[] strings = xml.split("\\{0\\}"); //$NON-NLS-1$
		para.appendChild(doc.createTextNode(strings[0]));
		para.appendChild(link);
		para.appendChild(doc.createTextNode(strings[1]));
	}


	private boolean checkTarget(List workItems, IProcessArea processArea, IProgressMonitor monitor) throws TeamRepositoryException {
		int numWorkItems = workItems.size();
		if (VAL_OPTIONAL.equals(fTarget) || numWorkItems == 0) {
			return true;
		}
		monitor.beginTask(Messages.getString("RequireWorkItemAdvisor.21"), numWorkItems); //$NON-NLS-1$
		try {
			for (Iterator e = workItems.iterator(); e.hasNext();) {
				IWorkItem workItem = (IWorkItem) e.next();
				if (hasRequiredTarget(workItem, processArea, new SubProgressMonitor(monitor, 1))) {
					return true;
				}
			}
		} finally {
			monitor.done();
		}

		return false;
	}

	private boolean hasRequiredTarget(IWorkItem workItem, IProcessArea processArea, IProgressMonitor monitor) throws TeamRepositoryException {
		return RequireWorkItemAdvisor.hasRequiredTarget(items, VAL_REQUIRED.equals(fTarget), workItem, processArea, monitor);
	}

	private boolean checkOwner(List workItems, IProgressMonitor monitor) throws TeamRepositoryException {
		int numWorkItems = workItems.size();
		if (VAL_OPTIONAL.equals(fOwner) || numWorkItems == 0) {
			return true;
		}
		monitor.beginTask(Messages.getString("RequireWorkItemAdvisor.27"), numWorkItems * 1000); //$NON-NLS-1$
		try {
			for (Iterator e = workItems.iterator(); e.hasNext();) {
				IWorkItem workItem = (IWorkItem) e.next();
				if (hasRequiredOwner(workItem, new SubProgressMonitor(monitor, 1000))) {
					return true;
				}
			}
		} finally {
			monitor.done();
		}

		return false;
	}

	private boolean hasRequiredOwner(IWorkItem workItem, IProgressMonitor monitor) throws TeamRepositoryException {
		IContributorHandle handle = workItem.getOwner();
		if (handle == null) {
			return false;
		}

		IContributor owner = (IContributor) items.fetchItem(handle, monitor);
		if (VAL_REQUIRED.equals(fOwner)) {
			return isValidOwner(owner);
		}
		return items.getLoggedInContributor().sameItemId(owner);
	}

	private boolean isValidOwner(IContributor owner) {
		if (owner == null) {
			return false;
		}
		String userId = owner.getUserId();
		return (userId != null) && (userId.trim().length() > 0) && (!Messages.getString("RequireWorkItemAdvisor.32").equalsIgnoreCase(userId));  //$NON-NLS-1$
	}

	public String getRequires() {
		return fRequires;
	}

	public String getTarget() {
		return fTarget;
	}	

}
