package com.ibm.stg.rtc.ext.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemType;

/**
 * This advisor synchronizes all custom attributes of every work item.
 * It should only add missing attributes and not remove attributes.
 * Note that advisors usually should not modify a work item, however this is a special 
 * case because we need to avoid sending multiple notifications. 
 * 
 * @author Kyle Christianson
 *
 */
public class SynchronizeAttributes extends AbstractService implements IOperationAdvisor {

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		IWorkItemType type = workItemCommon.findWorkItemType(newWorkItemState.getProjectArea(), newWorkItemState.getWorkItemType(), monitor);
		// pass null in the old work item type so this doesn't remove attributes
		workItemCommon.updateWorkItemType(newWorkItemState, type, null, monitor);

	}

}
