package com.ibm.stg.rtc.ext.participants;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.ChildWorkItemTypeStates;
import com.ibm.stg.rtc.ext.common.ChildWorkItemTypeStatesHelper;
import com.ibm.stg.rtc.ext.common.ConfigurationHelper;
import com.ibm.stg.rtc.ext.common.RepositoryCommonTasks;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.stg.rtc.ext.common.WorkItemCondition;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.TeamOperationCanceledException;
import com.ibm.team.process.common.advice.runtime.IOperationParticipant;
import com.ibm.team.process.common.advice.runtime.IParticipantInfoCollector;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.internal.WorkflowUtil;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.Identifier;
import com.ibm.team.workitem.common.workflow.IWorkflowAction;
import com.ibm.team.workitem.common.workflow.IWorkflowInfo;
import com.ibm.team.workitem.service.IWorkItemServer;

/**
 * Performs an action on a work item's parent if all of the parent's children work item conditions are met.
 * 
 * @author Karen Paone
 */

/* 
  	<followup-action id="com.ibm.stg.rtc.ext.participants.ActionOnParentWhenChildWorkItemInStatesParticipant" name="STG: Action on Parent when CRs in States" optional="false">	    
  		<!-- Perform resolve action on parent Task if 
	    	all child CRs are closed, no change, or canceled -->
		<workItemType id="task">
	    	<condition workItemAction="com.ibm.team.workitem.taskWorkflow.action.resolve">   
	        	<requiredChildType childWorkItemType="com.ibm.stg.workItemType.changeRecord">
	            	<state id="com.ibm.stg.workflow.changeRecord.state.noChange"/>
	            	<state id="com.ibm.stg.workflow.changeRecord.state.canceled"/>
	            	<state id="com.ibm.stg.workflow.changeRecord.state.closed"/>
	        	</requiredChildType>
	    	</condition>                            	
	    </workItemType>
		<workItemType id="com.ibm.stg.workItemType.stgDefect">
		    <!-- Perform fixed action on parent STG Defect if 
		    	all child STG Defects are closed or canceled and at least 1 in verify 
		    	AND all child CRs are closed, no change, or canceled and at least 1 in verify -->
	    	<condition workItemAction="com.ibm.stg.workflow.stgDefect.action.fixed">
	        	<requiredChildType childWorkItemType="com.ibm.stg.workItemType.stgDefect">
	            	<state requiredState="true" id="com.ibm.stg.workflow.stgDefect.state.verify"/>
	            	<state id="com.ibm.stg.workflow.stgDefect.state.closed"/>
	            	<state id="com.ibm.stg.workflow.stgDefect.state.canceled"/>
	        	</requiredChildType>
	        	<requiredChildType childWorkItemType="com.ibm.stg.workItemType.changeRecord">
	            	<state id="com.ibm.stg.workflow.changeRecord.state.closed"/>
	            	<state id="com.ibm.stg.workflow.changeRecord.state.noChange"/>
	            	<state id="com.ibm.stg.workflow.changeRecord.state.canceled"/>
	            	<state requiredState="true" id="com.ibm.stg.workflow.changeRecord.state.verify"/>
	        	</requiredChildType>
	    	</condition>
		    <!-- Perform fixed action on parent STG Defect if 
		    	all child CRs are no change or canceled and at least 1 in closed -->
	    	<condition workItemAction="com.ibm.stg.workflow.stgDefect.action.fixed">
	        	<requiredChildType childWorkItemType="com.ibm.stg.workItemType.changeRecord">
	                <state requiredState="true" id="com.ibm.stg.workflow.changeRecord.state.closed"/>
	                <state id="com.ibm.stg.workflow.changeRecord.state.noChange"/>
	                <state id="com.ibm.stg.workflow.changeRecord.state.canceled"/>
	        	</requiredChildType>
	    	</condition>     
	    </workItemType>
	</followup-action>
 */
public class ActionOnParentWhenChildWorkItemInStatesParticipant extends AbstractService implements
IOperationParticipant {

	public static final String ID = "com.ibm.stg.rtc.ext.participants.ActionOnParentWhenChildWorkItemInStatesParticipant";
	public static final String NAME = "STG: Action on Parent when Children Work Items in States";

	public void run(AdvisableOperation operation, IProcessConfigurationElement participantConfig,
			IParticipantInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;
		IWorkItem oldWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		}
		else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		}
		else {
			return;
		}

		tempObj = saveParameter.getOldState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			oldWorkItemState = (IWorkItem) tempObj;
		}

		// if existing work item and the state did not change - no further
		// processing needed
		if (oldWorkItemState != null && newWorkItemState.getState2().equals(oldWorkItemState.getState2())) {
			return;
		}

		IAuditableCommon auditableCommon = this.getService(IAuditableCommon.class);

		// retrieve the parent work item
		IWorkItem parent = workItemCommonTasks.getParentWorkItem(saveParameter.getNewReferences(), auditableCommon,
				monitor);
		if (parent == null) {
			return;
		}

		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		IWorkflowInfo workflowInfo = workItemCommon.findWorkflowInfo(parent, monitor);

		// determine what is configured for the parent work item type when children in states
		IProcessConfigurationElement typeConfiguration = ConfigurationHelper.getWITypeConfiguration(participantConfig,
				parent.getWorkItemType());
		Set<WorkItemCondition> conditions = ConfigurationHelper.getAllConditions(typeConfiguration);

		// get all the children
		List<IWorkItem> children = workItemCommonTasks.getAllChildWorkItems(parent,
				workItemCommon.getAuditableCommon(), workItemCommon, monitor);

		Set<String> parentActions = new HashSet<String>();
		for (WorkItemCondition condition : conditions) {
			// is there any action that can be made on the parent for this
			// condition
			Identifier<IWorkflowAction> actionId = (new WorkItemCommonTasks()).getActionIdentifierFromString(
					workflowInfo, condition.getWorkItemAction());
			String targetState = workflowInfo.getActionResultState(actionId).getStringIdentifier();

			// process the children if the parent work item is not already in the target
			// state && if it is a valid action to take on the parent based on
			// the current state
			if (!parent.getState2().getStringIdentifier().equals(targetState)
					&& WorkflowUtil.validState(workflowInfo, parent, actionId.getStringIdentifier())) {				
				if (checkChildrenStates(parent, children, newWorkItemState, condition, workItemCommon, collector, monitor)) {
					parentActions.add(condition.getWorkItemAction());
				}
			}
		}

		// no conditions were met or no action needed on parent
		if (parentActions.isEmpty()) {
			return;
		}

		// check that there is only 1 action on the parent of the passed
		// conditions
		String parentActionID = null;

		for (String action : parentActions) {
			if (parentActionID == null) {
				parentActionID = action;
			}
			if (!parentActionID.equals(action)) {
				// not legal to have more than 1 action on the parent work item
				IReportInfo info = collector
						.createInfo("Invalid Configuration",
								"There should be one unique parent work item action configured for a set of children work item states.");
				info.setProblemObject(parent.getItemHandle());
				info.setSeverity(IProcessReport.WARNING);
				collector.addInfo(info);

			}
		}

		Identifier<IWorkflowAction> parentAction = (new WorkItemCommonTasks()).getActionIdentifierFromString(
				workflowInfo, parentActionID);
		try {
			IWorkItemServer workItemServer = this.getService(IWorkItemServer.class);
			workItemServer.saveWorkItem2((IWorkItem)parent.getWorkingCopy(), null, parentAction.getStringIdentifier());
		}
		catch (TeamOperationCanceledException e) {
			RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
			// failed preconditions
			IReportInfo info = collector.createInfo("Follow-up action failed", "The follow-up action could not '"
					+ workflowInfo.getActionName(parentAction) + "' parent work item " + parent.getId()
					+ " because a precondition failed with the following message: \n"
					+ repoCommon.getFirstErrorDescription(e));
			info.setProblemObject(parent.getItemHandle());
			// this warning will display, but the failed preconditions will cause the entire transaction to rollback
			// add this info as a WARNING and suppress the errors so this work item save operation can continue
			info.setSeverity(IProcessReport.WARNING);	
			collector.addInfo(info);

			repoCommon.suppressAllErrors(NAME, e);
		}
	}

	private boolean checkChildrenStates(IWorkItem parent, List<IWorkItem> allChildren, IWorkItem childWorkItem, WorkItemCondition condition,
			IWorkItemCommon workItemCommon, IParticipantInfoCollector collector, IProgressMonitor monitor)
					throws TeamRepositoryException {
		// get the child types which are required
		Set<ChildWorkItemTypeStates> requiredChildTypes;
		try {
			requiredChildTypes = ChildWorkItemTypeStatesHelper.getChildTypesAndStates(condition.getConditionElement());
		}
		catch (TeamRepositoryException e) {
			IReportInfo info = collector.createInfo("Invalid Configuration", e.getMessage());
			info.setProblemObject(parent.getItemHandle());
			info.setSeverity(IProcessReport.WARNING);
			collector.addInfo(info);
			return false;
		}	

		boolean childWorkItemTypeRequired = false;

		// loop through the types of the children
		for (ChildWorkItemTypeStates requiredChildType : requiredChildTypes) {
			List<IWorkItem> children = ChildWorkItemTypeStatesHelper.getChildrenOfType(allChildren, requiredChildType
					.getChildType());
			if (children.isEmpty()) {
				// no children of this required child type on this parent,
				// condition not met
				return false;
			}
			if (childWorkItem.getWorkItemType().equals(requiredChildType.getChildType())) {
				childWorkItemTypeRequired = true;
			}
			List<String> childStates = requiredChildType.getChildStates();

			Set<String> foundStates = new HashSet<String>();

			for (IWorkItem child : children) {
				// for each child of this type check if it is in a valid state
				String childState = child.getState2().getStringIdentifier();
				if (childStates.contains(childState)) {
					foundStates.add(childState);
				}
				else if (child.sameItemId(childWorkItem)) {
					// ensure the work item for this operation meets the state
					// conditions to trigger an action on the parent
					return false;
				}
				else {
					//condition not met
					return false;
				}
			}

			// for this type, determine if the required states condition has
			// been met
			for (String requireChildState : requiredChildType.getRequiredChildStates()) {
				if (!foundStates.contains(requireChildState)) {
					// required state is missing, condition not met
					return false;
				}
			}
		}

		// the work item type this operation is against was not a required child type
		if (!childWorkItemTypeRequired) {
			return false;
		}

		return true;
	}
}
