package com.ibm.stg.rtc.ext.participants;

import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.stg.rtc.ext.common.ApprovalContainer;
import com.ibm.stg.rtc.ext.common.ApprovalWorkItemCondition;
import com.ibm.stg.rtc.ext.common.ApprovalsHelper;
import com.ibm.stg.rtc.ext.common.ApproverListContainer;
import com.ibm.stg.rtc.ext.common.RepositoryCommonTasks;
import com.ibm.stg.rtc.ext.common.SCMHelper;
import com.ibm.stg.rtc.ext.common.WorkItemChangeInfo;
import com.ibm.stg.rtc.ext.common.WorkItemCommonTasks;
import com.ibm.team.links.common.IReference;
import com.ibm.team.process.common.IIteration;
import com.ibm.team.process.common.IIterationHandle;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.TeamOperationCanceledException;
import com.ibm.team.process.common.advice.runtime.IOperationParticipant;
import com.ibm.team.process.common.advice.runtime.IParticipantInfoCollector;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.repository.common.PermissionDeniedException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.service.IContributorService;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.repository.service.IRepositoryItemService;
import com.ibm.team.workitem.common.ISaveOperationParameter;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.IWorkItemCommon;
import com.ibm.team.workitem.common.model.IApproval;
import com.ibm.team.workitem.common.model.IApprovalDescriptor;
import com.ibm.team.workitem.common.model.IApprovals;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;
import com.ibm.team.workitem.service.IWorkItemServer;

public class CreateSTGApprovalsParticipant extends AbstractService implements IOperationParticipant {

	public static final String ID = "com.ibm.stg.rtc.ext.participants.CreateSTGApprovalsParticipant";
	public static final String NAME = "STG: Create STG Approvals";

	public void run(AdvisableOperation operation, IProcessConfigurationElement participantConfig, IParticipantInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		IWorkItemServer workItemServer = this.getService(IWorkItemServer.class);
		IRepositoryItemService repositoryItemService = this.getService(IRepositoryItemService.class);
		IContributorService contributorService = this.getService(IContributorService.class);
		IWorkItemCommon workItemCommon = this.getService(IWorkItemCommon.class);
		WorkItemCommonTasks workItemCommonTasks = new WorkItemCommonTasks();
		ISaveParameter saveParameter = null;
		IWorkItem newWorkItemState = null;

		Object tempObj = operation.getOperationData();
		if (workItemCommonTasks.isSaveParamterType(tempObj)) {
			saveParameter = (ISaveParameter) tempObj;
		} else {
			return;
		}

		tempObj = saveParameter.getNewState();
		if (workItemCommonTasks.isIWorkItemType(tempObj)) {
			newWorkItemState = (IWorkItem) tempObj;
		} else {
			return;
		}

		List<ApprovalContainer> configuredApprovals = null;
		try{
			configuredApprovals = ApprovalsHelper.getConfiguredApprovals(participantConfig, newWorkItemState, workItemCommon);//getApprovalsToCreate(participantConfig, saveParameter);
			// call these methods to validate the xml early, they will throw an exception if an error occurs
		} catch (Exception e) {
			// problem reading the configured approvals
			IReportInfo info = collector.createInfo("Invalid approval configuration", e.getMessage());
			info.setSeverity(IProcessReport.ERROR);
			collector.addInfo(info);
			return;
		}

		// get the work item change info
		WorkItemChangeInfo workItemChangeInfo = WorkItemChangeInfo.getWorkItemChangeInfo(saveParameter, workItemServer, monitor, null);

		// get a fresh working copy
		IWorkItem workItemWC = (IWorkItem)workItemServer.findWorkItemById(newWorkItemState.getId(), IWorkItem.FULL_PROFILE, null).getWorkingCopy();
		boolean approvalsModified = false;

		for(ApprovalContainer approvalContainer : configuredApprovals){

			String approvalName = approvalContainer.getName();
			// do not use the fresh working copy because other follow-up actions may have already updated
			// the work item and changed its state in the repository
			IApprovalDescriptor approvalDescriptor = ApprovalsHelper.findApprovalDescriptorByName(newWorkItemState, approvalName);
			IApprovalDescriptor createdApprovalDesc = null;

			// if the approval is configured to be created and it doesn't already exist, then create it
			if(approvalContainer.isCreate(saveParameter) && approvalDescriptor == null){
				// now make sure the approval hadn't been created in a different follow-up action chain
				createdApprovalDesc = ApprovalsHelper.findApprovalDescriptorByName(workItemWC, approvalName);
				if(createdApprovalDesc == null){
					// create the new approval descriptor
					createdApprovalDesc = workItemWC.getApprovals().createDescriptor(approvalContainer.getApprovalType(), approvalName);
					IIterationHandle iterationHandle = newWorkItemState.getTarget();
					// if planned for iteration is not set, use the current iteration
					if(iterationHandle == null && operation.getDevelopmentLine() != null){
						iterationHandle = operation.getDevelopmentLine().getCurrentIteration();
					}
					// if current iteration is not set, use the first iteration in the timeline
					if(iterationHandle == null && operation.getDevelopmentLine() != null){
						IIterationHandle[] iterations = operation.getDevelopmentLine().getIterations();
						if(iterations.length != 0){
							iterationHandle = iterations[0];
						}
					}
					try {
						IIteration iteration = (iterationHandle == null) ? null : (IIteration)repositoryItemService.fetchItem(iterationHandle, IRepositoryItemService.COMPLETE);
						createdApprovalDesc.setDueDate(approvalContainer.getDueDate(iteration));
					} catch (Exception e) {
						IReportInfo info = collector.createInfo("Error calculating approval due date", e.getMessage());
						info.setSeverity(IProcessReport.WARNING);
						collector.addInfo(info);
					}

					workItemWC.getApprovals().add(createdApprovalDesc);
					approvalsModified = true;
				}
			}

			if(createdApprovalDesc != null){
				approvalDescriptor = createdApprovalDesc;
			}else if(approvalDescriptor != null){
				// now use the working copy to add the approvers
				approvalDescriptor = ApprovalsHelper.findApprovalDescriptorByName(workItemWC, approvalName);
			}

			// if the approval descriptor exists, update it's approver list
			if(approvalDescriptor != null && isNewApproversAllowed(approvalContainer, saveParameter)){
				// get the the approver list
				ApproverListContainer approverList = null;
				try{
					approverList = ApprovalsHelper.getApproverList(operation.getProcessArea(), approvalContainer.getName(), newWorkItemState, workItemServer.getAuditableCommon(), workItemCommon);
				}catch (Exception e){
					IReportInfo info = collector.createInfo("Invalid approver list configuration", e.getMessage());
					info.setSeverity(IProcessReport.ERROR);
					collector.addInfo(info);
				}

				if(approverList != null){
					Set<String> approversToAdd;
					if(createdApprovalDesc != null){
						// if a the approval was just created, add all configured approvers
						approversToAdd = approverList.getAllApprovers(newWorkItemState, workItemServer, monitor);
					}else{
						// if the approval already exists, only add changed approvers
						approversToAdd = approverList.getChangedApprovers(workItemChangeInfo, workItemServer, monitor);
					}
					for(String user : approversToAdd){
						// fetch the user and make sure it exists
						IContributorHandle contributorHandle = contributorService.fetchContributorByUserId(user);
						if(contributorHandle != null){
							IApprovals approvals = workItemWC.getApprovals();
							if(approvals.contains(approvalDescriptor, contributorHandle) == false && workItemCommonTasks.isUnassignedContributor(contributorHandle, workItemServer.getAuditableCommon(), monitor) == false){
								IApproval approver = approvals.createApproval(approvalDescriptor, contributorHandle);
								approvals.add(approver);
								approvalsModified = true;
							}
						} else{
							IReportInfo info = collector.createInfo("Invalid approver list configuration", "The user id '" + user + "' specified in the approver list configuration could not be found in the repository.");
							info.setSeverity(IProcessReport.ERROR);
							collector.addInfo(info);
							continue;
						}
					}
				}else{
					String message = "An approver list for team '" + operation.getProcessArea().getName() + "'";
					if (SCMHelper.getInternalReleaseName(newWorkItemState, workItemCommon, monitor) != null) {
						message += ", approval '" + approvalContainer.getName() + "', and internal release '" + SCMHelper.getInternalReleaseName(newWorkItemState, workItemCommon, monitor) + "'";
					}
					message += " was not found. The approval has been created without any approvers.";
					IReportInfo info = collector.createInfo("Approver list not found.", message);
					info.setSeverity(IProcessReport.WARNING);
					collector.addInfo(info);
				}
			}


		}

		// save work item
		if(approvalsModified){
			try{
				// add save parameters here so PreventSTGApprovalChange knows this code modified the approval: owner/creator change, parent owner/creator change, etc
				workItemServer.saveWorkItem3(workItemWC, null, null, workItemChangeInfo.getChangeStringParameters(WorkItemChangeInfo.CHANGE_INFO_PARAMETER));
			}catch(TeamOperationCanceledException e){
				RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
				IReportInfo info = collector.createInfo("Follow-up action failed", "The create approvals follow-up could not complete because a precondition failed with the following message: \n" + repoCommon.getFirstErrorDescription(e));
				info.setProblemObject(workItemWC.getItemHandle());
				info.setSeverity(IProcessReport.WARNING);
				collector.addInfo(info);
				// suppress the create approvals errors so this save can continue
				repoCommon.suppressAllErrors(NAME, e);
			}
		}

		// if this work item has a duplicate of link and contains changes, save the duplicate of work item with the string save parameters
		// to kick off this follow-up action with that work item
		if(workItemChangeInfo.containsChanges() && workItemCommonTasks.hasDuplicateOfLink(saveParameter.getNewReferences(), null, workItemServer.getAuditableCommon(), monitor)){
			// a duplicate of link exists, get this work item's change info and the duplicate of work item

			IWorkItem duplicateOfWI = workItemCommonTasks.getDuplicateOfWorkItem(saveParameter.getNewReferences(), workItemServer.getAuditableCommon(), monitor);

			// only save if the duplicate of work item is not already being saved otherwise a stale data exception will occur
			if(!isSaving(duplicateOfWI, saveParameter.getSaveOperationParameter())){
				// there are changes so save the duplicate work item with the save parameters

				IWorkItem duplicateOfWC = (IWorkItem)workItemServer.findWorkItemById(duplicateOfWI.getId(), IWorkItem.FULL_PROFILE, null).getWorkingCopy();
				try{
					workItemServer.saveWorkItem3(duplicateOfWC, null, null, workItemChangeInfo.getChangeStringParameters(WorkItemChangeInfo.CHANGE_DUPLICATE_INFO_PARAMETER));
				}catch(TeamOperationCanceledException e){
					RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
					IReportInfo info = collector.createInfo("Follow-up action failed", "The create approvals follow-up could not complete because a precondition failed with the following message: \n" + repoCommon.getFirstErrorDescription(e));
					info.setProblemObject(workItemWC.getItemHandle());
					info.setSeverity(IProcessReport.WARNING);
					collector.addInfo(info);
					// suppress the create approvals errors so this save can continue
					repoCommon.suppressAllErrors(NAME, e);
				}
			}
		}

		// if this work item contains parent info change parameters and contains children references, save the child work item with the
		// string save parameters
		// to kick off this follow-up action with that work item
		if (workItemChangeInfo.containsChanges() && saveParameter.getNewReferences().hasReferences(WorkItemEndPoints.CHILD_WORK_ITEMS)) {
			for (IReference childReference : workItemCommonTasks.getAllChildWorkItemReferences(saveParameter.getNewReferences(),
					workItemCommon.getAuditableCommon(), workItemCommon, monitor)) {
				IWorkItem childWorkItem = null;
				try {
					// a child work item link exists, get this work item's change info
					childWorkItem = (IWorkItem) workItemCommonTasks.getWorkItemFromReference(childReference,
							workItemCommon.getAuditableCommon(), monitor).getWorkingCopy();
				}
				catch (PermissionDeniedException e) {
					// skip updating this child work item
					continue;
				}
				if (!isSaving(childWorkItem, saveParameter.getSaveOperationParameter())) {
					IWorkItem childWC = (IWorkItem) workItemServer.findWorkItemById(childWorkItem.getId(), IWorkItem.FULL_PROFILE, null)
							.getWorkingCopy();
					try {
						workItemServer.saveWorkItem3(childWC, null, null,
								workItemChangeInfo.getChangeStringParameters(WorkItemChangeInfo.CHANGE_PARENT_INFO_PARAMETER));
					}
					catch (TeamOperationCanceledException e) {
						RepositoryCommonTasks repoCommon = new RepositoryCommonTasks();
						IReportInfo info = collector.createInfo("Follow-up action failed",
								"The create approvals follow-up could not complete because a precondition failed with the following message: \n"
										+ repoCommon.getFirstErrorDescription(e));
						info.setProblemObject(workItemWC.getItemHandle());
						info.setSeverity(IProcessReport.WARNING);
						collector.addInfo(info);
						// suppress the create approvals errors so this save can continue
						repoCommon.suppressAllErrors(NAME, e);
					}
				}

			}
		}
	}

	/**
	 * returns true if this save operation is saving the given work item
	 * @param workItem
	 * @param saveOperationParameter
	 * @return
	 */
	private boolean isSaving(IWorkItem workItem, ISaveOperationParameter saveOperationParameter){
		List<ISaveParameter> allSaveParameters = saveOperationParameter.getSaveParameters();
		for(ISaveParameter saveParameter : allSaveParameters){
			if(saveParameter.getNewState().sameItemId(workItem)){
				return true;
			}
		}
		return false;
	}

	private boolean isNewApproversAllowed(ApprovalContainer approvalContainer, ISaveParameter saveParameter) {
		for (ApprovalWorkItemCondition condition : approvalContainer.getConditionConfigs(saveParameter)) {
			if (condition.isModifyApprovals()) {
				return true;
			}
		}
		return false;
	}
}
