
'''
Created on Mar 13, 2013

@author: jdevries
'''

'''
TODO:

'''

'''
Future:
 - WI approvals (see note from Kyle)
 - clearly split general OSLC and RTC-specific REST calls (future: provide generic OSLC command line that would work with more than just RTC?)
 - WI History - investigate this: https://jazz.net/forum/questions/88945/how-to-get-the-history-of-a-work-item-by-oslc


Done:
 - review Tim's test questions to see if there are other error handling problems in this code
 - Query support
 - Session expired? would be nice to gracefully recover from an expired session... need to test
 - add comment, subscriber
 - list of queries owned by user in project area: <repo url>/oslc/queries.xml?oslc_cm.query=rtc_cm%3AprojectArea%3D%22_gbb_0EqVEd6HXO10niqZpg%22%20and%20dc%3Acreator%3D%22%7BcurrentUser%7D%22
  - query keys: dc:title name of query, rtc_cm:projectArea context id, dc:creator who it belongs to {currentUser}  should work for current user?
  - can we limit returned values so we don't load all the results? oslc_cm.properties=dc:title,dc:description
 - translation layer
  - investigate options for getting process template info
   - Attributes: OSLC attribute keys are not stored in process template so need some other way to identify those
   - Attribute values: most can be resolved by loading URL. this could be slow, need a cache system
   - need attribute type map to know whether or not the value should be translated
 - Cannot completely empty a list of all links
 - translation layer
 - RTCSession split out WI actions and environment investigation like factory URIs, process config info (different modules?)
 - run a query: /oslc/queries/<query key>/rtc_cm:results -- would be good if we could actually have the result fields match what is defined in the query
 - make serialization more robust, more testing (more than one repository): switched to shelve, need more testing
 - Error handling:
  - bad project name

Notes:
attr value notes:
*project area: load URL, use dc:title
progress tracking: what is this?
*priority: load URL, use dc:title
*plannedfor: load URL, use dc:title
*category/filedagainst: load URL, dc:title
estimates: in milliseconds, not really useful
users: fix problem talking to JTS and can include more data, or strip URL to get userid
*links: oslc_cm:label is useful text, rdf:resource is link to item
*comments: follow URL, dc:description is comment text, dc:creator has rdf:resource link to author, dc:created has create time
*type: follow URL, dc:title is name
'''

import xml.etree.ElementTree as ET
from xml.dom import minidom
#from urlparse import urlparse
import requests
from CLMException import CLMException
import json
import logging
from errormessages import getmsg
import datetime
import zlib


class JazzSession:
    shell = None
    repository = ''
    serviceProviders = None
    parsedServiceProviders = None
    #factoryURI = ''
    projectInfo = {}
    coreheaders ={}
    jsonheaders = {'User-Agent': 'jazzrest', 'Accept': 'application/json', 'Content-Type': 'application/json', 'OSLC-Core-Version': '2.0'}
    jsonoslc1headers = {'User-Agent': 'jazzrest', 'Accept': 'application/json', 'Content-Type': 'application/json', 'OSLC-Core-Version': '1.0'}
    rdfheaders = {'User-Agent': 'jazzrest', 'Accept':'application/rdf+xml', 'OSLC-Core-Version': '2.0'}
    logger = None
    
    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['logger']
        del odict['shell']

        return odict
    
    def __setstate__(self,d):
        self.__dict__.update(d)
        self.logger=logging.getLogger('mainlogger')
        self.shell=None
    
    def __init__(self):
        self.logger = logging.getLogger('mainlogger')
        self.checkzlib()
    
    def checkzlib(self):
        self.logger.debug('zlib version: '+zlib.ZLIB_VERSION)
        parts = zlib.ZLIB_VERSION.split('.')
        setencoding = False
        if len(parts)>1:
            maj = int(parts[0])
            minor = int(parts[1])
            if maj == 0:
                setencoding = True
            elif maj == 1:
                if minor < 2:
                    setencoding = True
        else:
            setencoding = True # just to be safe
        
        if setencoding:
            self.coreheaders = {'Accept-Encoding': 'bzip'}
            self.logger.debug('Found old version of zlib, forcing bzip: '+repr(self.coreheaders))
            
            self.jsonheaders.update(self.coreheaders)
            self.jsonoslc1headers.update(self.coreheaders)
            self.rdfheaders.update(self.coreheaders)
    
    def login(self,repo,user,password):
        aurl = repo+'/authenticated/identity'
        self.logger.debug('Checking URL: '+aurl)
        self.requestSession.verify=False
        authvalues = {'j_username': user, 'j_password': password}
        try:
            self.requestSession.get(aurl, verify=False, headers=self.jsonheaders)
            c = self.requestSession.post(repo+'/j_security_check', params=authvalues, verify=False, headers=self.jsonheaders)
        except:
            raise CLMException(*getmsg("RJ01001"))
        
        if c.status_code != 200:
            raise CLMException(*getmsg("RJ02001",status_code=c.status_code))
        self.logger.debug('Login status code: '+repo+' '+str(c.status_code))
        #self.logger.debug(c.text)
        authmsg = c.headers.get('X-com-ibm-team-repository-web-auth-msg','None')
        if len(c.cookies) == 0 or authmsg == 'authfailed': 
            raise CLMException(*getmsg("RJ03001",repository=repo,user=user))
        #self.logger.debug('authmsg: '+authmsg)
        self.logger.debug('session cookie count: '+str(len(self.requestSession.cookies)))
    
    def loginNew(self,repo,user,password):
        self.repository=repo
        self.requestSession = requests.Session()
        self.login(repo,user,password)


    def whoami(self):
        self.logger.debug('session cookie count: '+str(len(self.requestSession.cookies)))        
        resp = self.get(self.repository+'/whoami',headers=self.jsonheaders)

        self.logger.debug(resp.status_code)
        self.logger.debug("'"+resp.text+"'")

        return resp.text

    def ping(self):
        rval = False
        self.logger.debug('Enter Ping!')
        resp = self.requestSession.get(self.repository+'/whoami', self.jsonheaders)
        if resp.status_code != 200:
            rval = False
        else:
            authmsg = resp.headers.get('X-com-ibm-team-repository-web-auth-msg',None)
            if 'authrequired' == authmsg:
                self.logger.debug('Ping authrequired')
                rval = False
            else:
                rval = True
        self.logger.debug('Exit Ping!')
        return rval

    def getmessage(self,jsontext,defmessage=""):
        try:
            mo = json.loads(jsontext)
            return mo.get('oslc:message',defmessage)
        except:
            pass
        return defmessage

class JTSSession(JazzSession):    
    ''' JTS specific stuff '''
    def postServerMessageJSON(self, message):
        self.logger.debug(self.repository+'/serverMessages')
        r = self.requestSession.post(self.repository+'/serverMessages', data=message, headers={'Accept': 'text/json', 'Content-Type': 'application/json'}, verify=False)
        return r
        
    def postServerMessage(self, text, start_date, end_date):
        message = {}
        message['description']=text
        message['startDate']=start_date
        message['endDate']=end_date
        messagestr = json.dumps(message)

        r = self.postServerMessageJSON(messagestr)
        if r.status_code == 201:
            return json.loads(r.text)
        else:
            self.logger.debug(r.text)
            return {'error': 'check the debug'}
    
    def getServerMessagesJSON(self):
        r = self.requestSession.get(self.repository+'/serverMessages', headers={'Accept': 'text/json', 'Content-Type': 'application/json'}, verify=False)
        if r.status_code == 200:
            return json.loads(r.text)
        else:
            self.logger.debug(r.text)
            return {'error': 'check the debug'}

class RTCSession(JazzSession):
    
    jtsurl = ''
    
    def __init__(self):
        #super(RTCSession, self).__init__()
        JazzSession.__init__(self)
        self.logger.debug('init coreheaders = '+repr(self.coreheaders))
        
    
    def loginNew(self,repo,user,password):
        #super(RTCSession, self).loginNew(repo,user,password)
        JazzSession.loginNew(self,repo,user,password)
        self.prepareServices()
        #Log in to JTS, too
        
        self.logger.debug('log in to JTS: '+self.jtsurl)
        self.login(self.jtsurl,user,password)
        #self.logger.debug(self.whoami())
    
    ''' RTC Specific Stuff ''' 
        
    def getText(self, node):
        rval = ''
        for child in node.childNodes:
            if child.nodeType == child.TEXT_NODE:
                rval = rval+child.data
        
        return rval

    def prepareServices(self):
        self.logger.debug('enter prepare services')
        sdresp = self.get(self.repository + '/rootservices',headers=self.rdfheaders)
        #self.logger.debug(sdresp.text)
        #self.logger.debug(self.serviceDoc)
        #root = ET.fromstring(self.serviceDoc)
        #self.logger.debug(ET._namespace_map)
        root = minidom.parseString(sdresp.text)
        atEl = root.getElementsByTagName('dc:title')
        appname = ''
        if len(atEl)>0: appname = self.getText(atEl[0])
        self.logger.debug(appname)
        if appname == 'Change and Configuration Management':
            self.setjtsurl(root)
            el = root.getElementsByTagName('oslc_cm:cmServiceProviders')
            curi = el[0].attributes['rdf:resource'].value
            #self.logger.debug(curi)
            cresp = self.get(curi,headers=self.jsonheaders)
            self.serviceProviders = json.loads(cresp.text)
            
    def setjtsurl(self,rootel):
        el = rootel.getElementsByTagName('jfs:users')
        uuri = el[0].attributes['rdf:resource'].value
        if uuri.endswith('/users'):
            self.jtsurl = uuri[:-6]
            self.logger.debug(self.jtsurl)


    def getworkitembyid(self, wiId,mask=None):
        self.logger.debug('Getting: '+wiId)
        url = self.repository+'/oslc/workitems/'+str(wiId)
        if mask!=None:
            url = url + mask
        r = self.get(url, headers=self.jsonheaders)
        if r.status_code != 200:
            raise CLMException(*getmsg("RJ02002",id=wiId))
             
        self.logger.debug(repr(r))
        rwi = json.loads(r.text)
        return(r, rwi)


    def subscribe(self,wiId,subscribers):
        if len(subscribers)==0:
            subscribers = [self.whoami()]
        self.logger.debug('Subscribing to work item: '+wiId+' :: '+str(subscribers))

        (_, rwi) = self.getworkitembyid(wiId,'?oslc_cm.properties=rtc_cm:subscribers')
        
        slist = rwi['rtc_cm:subscribers']
        for sub in subscribers:
            td = {}
            td['rdf:resource']=sub
            slist.append(td)
        
        content = json.dumps(rwi)
        r = self.put(self.repository+'/oslc/workitems/'+str(wiId)+'?oslc_cm.properties=rtc_cm:subscribers', data=content, headers=self.jsonheaders)
        if r.status_code != 200:
            self.logger.debug('status code: '+str(r.status_code))
            raise CLMException(*getmsg("RJ02003",message=self.getmessage(r.text,'')))
        rwi = json.loads(r.text)
        return (r,rwi)

    def unsubscribe(self,wiId,subscribers):
        if len(subscribers)==0:
            subscribers = [self.whoami()]

        self.logger.debug('Unsubscribing from work item: '+wiId+' :: '+str(subscribers))

        (_, rwi) = self.getworkitembyid(wiId,'?oslc_cm.properties=rtc_cm:subscribers')
        
        slist = rwi['rtc_cm:subscribers']
        nlist = []
        for sub in slist:
            us = sub['rdf:resource']
            if us not in subscribers:
                nlist.append(sub)
        rwi['rtc_cm:subscribers']=nlist
        
        content = json.dumps(rwi)
        r = self.put(self.repository+'/oslc/workitems/'+str(wiId)+'?oslc_cm.properties=rtc_cm:subscribers', data=content, headers=self.jsonheaders)
        if r.status_code != 200:
            raise CLMException(*getmsg("RJ02004",message=self.getmessage(r.text)))
        rwi = json.loads(r.text)
        return (r,rwi)

    def addcomment(self,wiId,comment):
        self.logger.debug('Adding comment to work item: '+wiId+' :: '+comment)

        # (code, rwi) = self.getworkitembyid(wiId)
        
        commentdict = {}
        commentdict['dcterms:description'] = comment
        
        
        #commentdict = {'oslc:comment':[{'dcterms:description':comment}]}
        
        content = json.dumps(commentdict)
        
        uri = self.repository+'/oslc/workitems/'+str(wiId)+'/rtc_cm:comments/oslc:comment'
        
        self.logger.debug('posting to: '+uri)
        
        r = self.post(uri, data=content, headers=self.jsonheaders)
        if r.status_code != 201:
            self.logger.debug('Status code: '+str(r.status_code))
            problem = self.getmessage(r.text)
            raise CLMException(*getmsg("RJ02005",message=problem))
        rwi = json.loads(r.text)
        return (r,rwi)
        
    def link(self,wiId,linktype,linkdest):
        self.logger.debug('Adding links to work item: '+wiId+' :: '+linktype+' > '+str(linkdest))

        # (code, rwi) = self.getworkitembyid(wiId)
        linklist = []
        for ld in linkdest:
            linkdict = {}
            linkdict['rdf:resource'] = ld
            linklist.append(linkdict)
        
        content = json.dumps(linklist)
        
        r = self.post(self.repository+'/oslc/workitems/'+str(wiId)+'/'+linktype, data=content, headers=self.jsonheaders)
        if r.status_code != 201:
            self.logger.debug('Status code: '+str(r.status_code))
            problem = self.getmessage(r.text)
            raise CLMException(*getmsg("RJ02006",message=problem))
        rwi = json.loads(r.text)
        return (r,rwi)

    def unlink(self,wiId,linktype,linkdest):
        self.logger.debug('Removing links from work item: '+wiId+' :: '+linktype+' > '+str(linkdest))

        (_, rwi) = self.getworkitembyid(wiId)

        clinks = rwi[linktype]
        self.logger.debug('Unlink starting length: '+str(len(clinks)))
        linklist = []
        for cl in clinks:
            rn = cl['rdf:resource']
            if rn not in linkdest:
                linklist.append(cl)
        self.logger.debug('Unlink remaining length: '+str(len(linklist)))

        r = None

        if len(linklist)>0:
            content = json.dumps(linklist)
            r = self.put(self.repository+'/oslc/workitems/'+str(wiId)+'/'+linktype, data=content, headers=self.jsonheaders)
        else:
            nwi = {linktype: []}
            #rwi[linktype]=linklist
            content = json.dumps(nwi)
            r = self.put(self.repository+'/oslc/workitems/'+str(wiId), data=content, headers=self.jsonheaders)
        if r.status_code != 201:
            self.logger.debug('Status code: '+str(r.status_code))
            problem = self.getmessage(r.text)
            raise CLMException(*getmsg("RJ02007",message=problem))
        rwi = json.loads(r.text)
        return (r,rwi)

    def modifyworkitem(self,wi,wiId,etag,action=None):
        #wiId = wi['dc:identifier']
        if action!=None:
            self.logger.debug("changing state: "+action)
        #self.logger.debug(content)
        uri = self.repository+'/oslc/workitems/'+str(wiId)
        if action != None:
            uri = uri + '?_action='+action
        #self.logger.debug(r.headers)
        comment = wi.get('oslc:discussedBy',{}).get('comment',None)
        newh = self.jsonheaders.copy()
        newh['If-Match'] = etag
        #self.logger.debug(newh)
        content = json.dumps(wi)
        r = self.put(uri, data=content, headers=newh)
        if r.status_code != 200:
            self.logger.debug('Status code: '+str(r.status_code))
            problem = self.getmessage(r.text)
            raise CLMException(*getmsg("RJ02008",message=problem))

        if comment != None:
            self.addcomment(wiId, comment)
            (_,rwi)=self.getworkitembyid(wiId)
        else:
            rwi = json.loads(r.text)
        
        if action!=None:
            # check whether action happened
            pass
        return(r, rwi)

    def createworkitem(self, wijson, project, witype):
        uri = self.getFactoryURIs(project)[witype]
        #self.logger.debug(uri)
        if 'dcterms:type' not in wijson:
            wijson['dcterms:type']=witype
        
        comment = wijson.get('oslc:discussedBy',{}).get('comment',None)

        
        content = json.dumps(wijson,sort_keys=True,indent=4,separators=(',',': '))
        self.logger.debug(content)
        r = self.post(uri, data=content, headers=self.jsonheaders)
        if r.status_code != 201:
            self.logger.debug('Status code: '+str(r.status_code))
            problem = self.getmessage(r.text)
            raise CLMException(*getmsg("RJ02009",message=problem))
        
        rwi = json.loads(r.text)
        
        if comment != None:
            wiId = rwi.get("dcterms:identifier",'')
            self.addcomment(wiId, comment)
            (_,rwi)=self.getworkitembyid(wiId)
        else:
            rwi = json.loads(r.text)


        return(r, rwi)

    def readProjectServices(self, provider):
        project = provider.get('dcterms:title','NO TITLE')
        detailsURI = provider.get('oslc:details',[{}])[0].get('rdf:resource','')
        servicesURI = provider.get('rdf:about','')
        
        return (project, detailsURI, servicesURI)

    def loadProjectInfo(self):
        self.logger.debug('enter load project info')
        #self.logger.debug(self.serviceProviders)
        # self.parsedServiceProviders
        providers = self.serviceProviders.get('oslc:serviceProvider',{})
        for provider in providers:
            (project, detailsURI, servicesURI) = self.readProjectServices(provider)
            
            self.projectInfo[project] = {'detailsURI': detailsURI, 'servicesURI': servicesURI}
            self.projectInfo[project]['contextId'] = detailsURI[detailsURI.rfind('/')+1:]
    
    def loadFactoryURIs(self, root):
        factories = {}
        services = root.get('oslc:service',[])
        fl = []
        for s in services:
            if 'oslc:creationFactory' in s:
                fl = s.get('oslc:creationFactory')
        for f in fl:
            key = f.get('oslc:resourceShape',{}).get('rdf:resource','abcd/default.changerequest').split('/')[-1]
            url = f.get('oslc:creation',{}).get('rdf:resource','')
            factories[key]=url
        
        return factories
    
    def loadProjectServices(self,projectInfo):
        self.logger.debug('enter load project services')
        su = projectInfo['servicesURI']
        #self.logger.debug(su)
        resp = self.get(su, headers=self.jsonheaders)
        #self.logger.debug(resp.text)
        serviceDoc = json.loads(resp.text)
        projectInfo['factoryURIs'] = self.loadFactoryURIs(serviceDoc)
        
    
    def getFactoryURIs(self, project):
        if project not in self.projectInfo:
            self.loadProjectInfo()
        if project not in self.projectInfo:
            if project == None:
                raise CLMException(*getmsg("RJ03005"))
            else:
                raise CLMException(*getmsg("RJ03002",project=project))
        pi = self.projectInfo[project]
        if 'factoryURIs' not in pi:
            self.loadProjectServices(pi)
            
        return self.projectInfo[project]['factoryURIs']
    
    def getWorkItemTypes(self,project):
        return self.getFactoryURIs(project)
    
    def listqueries(self,project,name=None,scope="personal"):
        pi = self.getprojectuuid(project)
            
        values = {}
        values['oslc_cm.properties']='dc:title,dc:description'
        qstr = 'rtc_cm:projectArea="'+pi+'"'
        if scope == 'personal':
            qstr = qstr + ' and dc:creator="{currentUser}"'
        if name != None:
            qstr = qstr + ' and dc:title="'+name+'"'
        values['oslc_cm.query']=qstr

        qr = self.get(self.repository+'/oslc/queries',params=values,headers=self.jsonoslc1headers)
        if qr.status_code != 200:
            self.logger.debug('Status code: '+str(qr.status_code))
            problem = self.getmessage(qr.text)
            raise CLMException(*getmsg("RJ02010",message=problem))
        return (qr, json.loads(qr.text))
    '''
old run query    
    def runquery(self,project,query,scope="personal"):
        (_,ql) = self.listqueries(project, query, scope)
        self.logger.debug(ql)
        values = {}
        values['oslc_cm.properties']='dc:title,dc:identifier'
        rl = ql.get('oslc_cm:results',[])
        url = None
        if len(rl)>0:
            url = rl[0].get('rdf:resource',None)
        if url == None:
            raise CLMException(*getmsg("RJ03201",query=query))
        key = url[url.rfind('/')+1:]
        qr = self.get(self.repository+'/oslc/queries/'+key+'/rtc_cm:results',params=values,headers=self.jsonoslc1headers)
        if qr.status_code != 200:
            self.logger.debug('Status code: '+str(qr.status_code))
            problem = self.getmessage(qr.text)
            raise CLMException(*getmsg("RJ02011",message=problem))        
        return(qr, json.loads(qr.text))
    '''
      
    def runquery(self,project,query,scope="personal"):
        (_,ql) = self.listqueries(project, query, scope)
        self.logger.debug(ql)
        values = {}
        values['oslc_cm.properties']='dc:title,dc:identifier'
        rl = ql.get('oslc_cm:results',[])
        url = None
        if len(rl)>0:
            url = rl[0].get('rdf:resource',None)
        if url == None:
            raise CLMException(*getmsg("RJ03201",query=query))
        key = url[url.rfind('/')+1:]
        startIndex = 0
        maxResults = 251
        hasmore = True
        headers = None
        reslist = []
        while hasmore:
            content='startIndex={2}&maxResults={3}&filterAttribute=&filterValue=&itemId={0}&projectAreaItemId={1}'.format(key,self.getprojectuuid(project),startIndex,maxResults)
            postqry=self.repository+'/service/com.ibm.team.workitem.common.internal.rest.IQueryRestService/getResultSet?'+content
            qr = self.post(postqry,headers=self.jsonheaders)
    
            if qr.status_code != 200:
                self.logger.debug('Status code: '+str(qr.status_code))
                problem = self.getmessage(qr.text)
                raise CLMException(*getmsg("RJ02011",message=problem))
            (tc,lres) = self.parsequeryresult(qr.text)
            startIndex = startIndex + maxResults
            if headers==None and 'headers' in lres:
                headers=lres['headers']
            reslist = reslist + lres['results']
            if startIndex>=tc or startIndex>=1000:
                hasmore = False
        qres = {'results':reslist,'headers':headers}
        
        return(qr, qres)
        #getqry='https://jazzt1.rchland.ibm.com:11443/ccm/service/com.ibm.team.workitem.common.internal.rest.IQueryRestService/query?itemId=_RnVhwKegEeKRacWU_ckh9Q&includeCount=true&projectAreaItemId=_7Vg6sN2aEeGY5fdMWcTDTQ'
        #gqr = self.rsess.get(getqry,headers=self.jsonheaders)
        ''' this will get the query results as seen on the web client '''
#        content='startIndex=0&maxResults=50&filterAttribute=&filterValue=&itemId=_RnVhwKegEeKRacWU_ckh9Q&projectAreaItemId=_7Vg6sN2aEeGY5fdMWcTDTQ&jsonQuery={"name":"Active Work I Own","description":"hi!","itemId":"_RnVhwKegEeKRacWU_ckh9Q","csvExportLink":"/ccm/resource/itemOid/com.ibm.team.workitem.query.QueryDescriptor/_RnVhwKegEeKRacWU_ckh9Q?_mediaType=text/csv","htmlExportLink":"/ccm/resource/itemOid/com.ibm.team.workitem.query.QueryDescriptor/_RnVhwKegEeKRacWU_ckh9Q?_mediaType=text/html","projectAreaItemId":"_7Vg6sN2aEeGY5fdMWcTDTQ"}'

    def parsequeryresult(self,resxml):
        lres = {}
        root = ET.fromstring(resxml.encode('UTF-8'))
        for v in root.findall('.//value'):
            qres = []
            startIndexE = v.find('startIndex')
            totalCountE = v.find('totalCount')
            startIndex = int(startIndexE.text)
            totalCount = int(totalCountE.text)
            self.logger.debug('startIndex {0} totalcount {1}'.format(startIndex,totalCount))
            coltypes = []
            collabels = []
            for header in v.findall('headers'):
                label = header.find('label').text
                coltype = header.find('attributeType').text
                collabels.append(label)
                coltypes.append(coltype)
            #print(','.join(collabels))
            
            for row in v.findall('rows'):
                labels = row.findall('labels')
                rowd = {}
                cind = 0
                for l in labels:
                    val = l.text
                    if val != None and coltypes[cind]=='timestamp':
                        try:
                            iv = int(val)
                            val = self.formattimestamp(iv)
                        except:
                            pass
                    rowd[collabels[cind]]=val
                    cind=cind+1
                qres.append(rowd)
            lres['results']=qres
            lres['headers']=collabels
        return (totalCount,lres)

    def formattimestamp(self,ms):
        dt = datetime.datetime.fromtimestamp(ms/1000)
        return dt.isoformat()

    def getprojectuuid(self,project):
        self.logger.debug('Looking up: '+str(project))
        if project not in self.projectInfo:
            self.loadProjectInfo()
        if project not in self.projectInfo:
            if project==None:
                raise CLMException(*getmsg("RJ03005",project=project))
            raise CLMException(*getmsg("RJ03002",project=project))
        pi = self.projectInfo[project]['contextId']
        return pi
    
    def get(self,url,**kwargs):
        try:
            r = self.requestSession.get(url,**kwargs)
        except:
            raise CLMException(*getmsg("RJ01001"))
        authmsg = r.headers.get('X-com-ibm-team-repository-web-auth-msg',None)
        attempts = 0
        while 'authrequired' == authmsg and attempts<3:
            if attempts > 0:
                self.shell.cpassword = None
            self.logger.debug('Need to authorize')
            self.shell.login(self,self.repository,self.shell.cuser,self.shell.cpassword)
            try:
                r = self.requestSession.get(url,**kwargs)
            except:
                raise CLMException(*getmsg("RJ01001"))
            authmsg = r.headers.get('X-com-ibm-team-repository-web-auth-msg',None)  
            attempts=attempts+1
        return r

    def put(self,url,**kwargs):
        try:
            r = self.requestSession.put(url,**kwargs)
        except:
            raise CLMException(*getmsg("RJ01001"))
        authmsg = r.headers.get('X-com-ibm-team-repository-web-auth-msg',None)
        attempts = 0
        while 'authrequired' == authmsg and attempts<3:
            if attempts > 0:
                self.shell.cpassword = None
            self.logger.debug('Need to authorize')
            self.shell.login(self,self.repository,self.shell.cuser,self.shell.cpassword)
            try:
                r = self.requestSession.put(url,**kwargs)
            except:
                raise CLMException(*getmsg("RJ01001"))
            authmsg = r.headers.get('X-com-ibm-team-repository-web-auth-msg',None)   
            attempts=attempts+1        
        return r

    def post(self,url,**kwargs):
        try:
            r = self.requestSession.post(url,**kwargs)
        except:
            raise CLMException(*getmsg("RJ01001"))
        authmsg = r.headers.get('X-com-ibm-team-repository-web-auth-msg',None)
        attempts = 0
        while 'authrequired' == authmsg and attempts<3:
            if attempts > 0:
                self.shell.cpassword = None
            self.logger.debug('Need to authorize')
            self.shell.login(self,self.repository,self.shell.cuser,self.shell.cpassword)
            try:
                r = self.requestSession.post(url,**kwargs)
            except:
                raise CLMException(*getmsg("RJ01001"))
            authmsg = r.headers.get('X-com-ibm-team-repository-web-auth-msg',None)   
            attempts=attempts+1       
        return r
